---
output: html_document
---




## Identifying cell-containing droplets/microwells

For droplet based methods only  a fraction of droplets contain both beads 
and an intact cell. However, biology experiments are messy and some RNA will 
leak out of dead/damaged cells. So droplets without an intact cell are likely to capture a small amount of the ambient RNA which will end up in the sequencing 
library and contribute a reads to the final sequencing output. The variation in
 droplet size, amplification efficiency, and sequencing will lead both 
"background" and real cells to have a wide range of library sizes. Various
approaches have been used to try to distinguish those cell barcodes which
correspond to real cells.

### 'Knee' point

One of the most used methods use the total molecules (could be applied to total reads) per 
barcode and try to find a "break point" between bigger libraries which are 
cells + some background and smaller libraries assumed to be purely background. 
Let's load some example simulated data which contain both large and small cells:


```r
umi_per_barcode <- read.table("data/droplet_id_example_per_barcode.txt.gz")
truth <- read.delim("data/droplet_id_example_truth.gz", sep=",")
```
__Exercise__
How many unique barcodes were detected? 
How many true cells are present in the data?
To simplify calculations for this section exclude all barcodes 
with fewer than 10 total molecules.

__Answer__


One approach is to look for the inflection point where the total molecules per 
barcode suddenly drops: 


```r
barcode_rank <- rank(-umi_per_barcode[,2])
plot(barcode_rank, umi_per_barcode[,2], xlim=c(1,8000))
```

<img src="cell-calling_files/figure-html/unnamed-chunk-4-1.png" width="90%" style="display: block; margin: auto;" />

Here we can see an roughly exponential curve of library sizes, so to make
things simpler lets log-transform them.


```r
log_lib_size <- log10(umi_per_barcode[,2])
plot(barcode_rank, log_lib_size, xlim=c(1,8000))
```

<img src="cell-calling_files/figure-html/unnamed-chunk-5-1.png" width="90%" style="display: block; margin: auto;" />

That's better, the "knee" in the distribution is much more pronounced. We 
could manually estimate where the "knee" is but it much more reproducible to
algorithmically identify this point.


```r
# inflection point
o <- order(barcode_rank)
log_lib_size <- log_lib_size[o]
barcode_rank <- barcode_rank[o]

rawdiff <- diff(log_lib_size)/diff(barcode_rank)
inflection <- which(rawdiff == min(rawdiff[100:length(rawdiff)], na.rm=TRUE))

plot(barcode_rank, log_lib_size, xlim=c(1,8000))
abline(v=inflection, col="red", lwd=2)
```

<img src="cell-calling_files/figure-html/unnamed-chunk-6-1.png" width="90%" style="display: block; margin: auto;" />

```r
threshold <- 10^log_lib_size[inflection]

cells <- umi_per_barcode[umi_per_barcode[,2] > threshold,1]
TPR <- sum(cells %in% truth[,1])/length(cells)
Recall <- sum(cells %in% truth[,1])/length(truth[,1])
c(TPR, Recall)
```

```
## [1] 1.0000000 0.7831707
```

### Mixture model

Another is to fix a mixture model and find where the higher and lower distributions intersect. However, data may not fit the assumed distributions very well:


```r
set.seed(-92497)
# mixture model
require("mixtools")
```

```
## Loading required package: mixtools
```

```
## mixtools package, version 1.1.0, Released 2017-03-10
## This package is based upon work supported by the National Science Foundation under Grant No. SES-0518772.
```

```r
mix <- normalmixEM(log_lib_size)
```

```
## number of iterations= 43
```

```r
plot(mix, which=2, xlab2="log(mol per cell)")
```

<img src="cell-calling_files/figure-html/unnamed-chunk-7-1.png" width="90%" style="display: block; margin: auto;" />

```r
p1 <- dnorm(log_lib_size, mean=mix$mu[1], sd=mix$sigma[1])
p2 <- dnorm(log_lib_size, mean=mix$mu[2], sd=mix$sigma[2])
if (mix$mu[1] < mix$mu[2]) {
	split <- min(log_lib_size[p2 > p1])
} else {
	split <- min(log_lib_size[p1 > p2])
}
```
__Exercise__
Identify cells using this split point and calculate the TPR and Recall.

__Answer__



### Expected Number of Cells

A third method used by CellRanger V2, assumes a ~10-fold range of library sizes for real 
cells and estimates this range using the expected number of cells.


```r
n_cells <- length(truth[,1])
# CellRanger v2
totals <- umi_per_barcode[,2]
totals <- sort(totals, decreasing = TRUE)
# 99th percentile of top n_cells divided by 10
thresh = totals[round(0.01*n_cells)]/10
plot(totals, xlim=c(1,8000))
abline(h=thresh, col="red", lwd=2)
```

<img src="cell-calling_files/figure-html/unnamed-chunk-9-1.png" width="90%" style="display: block; margin: auto;" />
__Exercise__
Identify cells using this threshodl and calculate the TPR and Recall.

__Answer__


### EmptyDroplets

Finally (EmptyDrops)[https://github.com/MarioniLab/DropletUtils] is what we recommend using in calling cell barcodes for droplet-based single-cell datasets. It should be noted that in `cellranger count v3`, `EmptyDroptlet` algoritms has been applied in their filtering for true cell barcode step. 

Instead of trying to find a 'threshold' in UMIs counts for determining true cells, `EmptyDroplet` uses the full genes x cells molecule count matrix for all droplets and estimates the profile of 
"background" RNA from those droplets with extremely low counts, then looks for
cells with gene-expression profiles which differ from the background. 

This is combined with an inflection point method since background RNA often looks very
similar to the expression profile of the largests cells in a population. As 
such EmptyDrops is the only method able to identify barcodes for very small
cells in highly diverse samples.

Below we have provided code for how this method is currently run:


```r
library("Matrix")
raw.counts <- readRDS("data/pancreas/muraro.rds")

library("DropletUtils")
example(write10xCounts, echo=FALSE) 
dir.name <- tmpdir
list.files(dir.name)
sce <- read10xCounts(dir.name)
sce
my.counts <- DropletUtils:::simCounts()
br.out <- barcodeRanks(my.counts)

# Making a plot.
plot(br.out$rank, br.out$total, log="xy", xlab="Rank", ylab="Total")
o <- order(br.out$rank)
lines(br.out$rank[o], br.out$fitted[o], col="red")

abline(h=metadata(br.out)$knee, col="dodgerblue", lty=2)
abline(h=metadata(br.out)$inflection, col="forestgreen", lty=2)
legend("bottomleft", lty=2, col=c("dodgerblue", "forestgreen"), 
    legend=c("knee", "inflection"))
# emptyDrops
set.seed(100)
e.out <- emptyDrops(my.counts)
is.cell <- e.out$FDR <= 0.01
sum(is.cell, na.rm=TRUE)

plot(e.out$Total, -e.out$LogProb, col=ifelse(is.cell, "red", "black"),
    xlab="Total UMI count", ylab="-Log Probability")

# plot(e.out$Total, -e.out$LogProb, col=ifelse(is.cell, "red", "black"),
#     xlab="Total UMI count", ylab="-Log Probability")
# 
# cells <- colnames(raw.counts)[is.cell]
# 
# TPR <- sum(cells %in% truth[,1])/length(cells)
# Recall <- sum(cells %in% truth[,1])/length(truth[,1])
# c(TPR, Recall)
```




