---
output:
  html_document: default
---





# Clustering and cell annotation

## Clustering Methods


Once we have normalized the data and removed confounders we can carry out analyses that are relevant to the biological questions at hand. The exact nature of the analysis depends on the dataset. Nevertheless, there are a few aspects that are useful in a wide range of contexts and we will be discussing some of them in the next few chapters. We will start with the clustering of scRNA-seq data.



### Introduction

One of the most promising applications of scRNA-seq is _de novo_ discovery and
annotation of cell-types based on transcription profiles. Computationally, this
is a hard problem as it amounts to __unsupervised clustering__. That is, we need
to identify groups of cells based on the similarities of the transcriptomes
without any prior knowledge of the labels. Moreover, in most situations we do
not even know the number of clusters _a priori_. The problem is made even more
challenging due to the high level of noise (both technical and biological) and
the large number of dimensions (i.e. genes).


When working with large datasets, it can often be beneficial to apply
some sort of dimensionality reduction method. By projecting
the data onto a lower-dimensional sub-space, one is often able to
significantly reduce the amount of noise. An additional benefit is
that it is typically much easier to visualize the data in a 2 or
3-dimensional subspace. We have already discussed PCA (chapter \@ref(visual-pca)) and t-SNE (chapter \@ref(visual-pca)).

 __Challenges in clustering__ 
 
* What is the number of clusters _k_?
* What defines a good clustering?
* What is a cell type?
* Scalability: in the last few years the number of cells in scRNA-seq experiments has grown by several orders of magnitude from ~$10^2$ to ~$10^6$


### Unsupervised clustering methods

Three main ingredients of a complete clustering method:

- __Measure of similarity__: how do we quantify how close two data points are? 

- __Quality function__: how do we decide how "good" is a clustering/partition? 

- __Algorithm__: how to find the clustering whose quality function is optimized? 



#### Hierarchical clustering

Hierarchical clustering is basically the only type of clustering algorithm that 
does not seek to optimize a quality function, because it builds a hierarchy of 
clusters, instead of one single clustering result as the output. 

There are two types of strategies: \
 - __Agglomerative__ (bottom-up): each observation starts in its own cluster, and pairs of clusters are merged as one moves up the hierarchy.\
 - __Divisive__ (top-down): all observations start in one cluster, and splits are performed recursively as one moves down the hierarchy.

<center>
![](figures/hierarchical_clustering1.png)     &nbsp;  ![](figures/hierarchical_clustering2.png){width=40%}
</centerr>

#### _k_-means clustering

__Measure of similarity__: Euclidean distance 

 __Quality function__: Within cluster distance 

 __Algorithm__:\
 ![](figures/k-means.png)

 __Advantage__: Fast

 __Drawbacks__:\
- Sensitive to initial clustering\
- Sensitive to outliers \
- Need to specify K\
- Tend to find clusters of similar sizes\

 __Tools related to K-means__:  [SC3](http://bioconductor.org/packages/SC3/)



#### Graph-based methods

Real world networks usualy display big inhomogeneities or __community structure__. 
Communities or clusters or modules are groups of vertices which probably share common properties and/or play similar roles whithin the graph. 
In recent years there has been a lot of interest in
detecting communities in networks in various domains.

<center>![](figures/network_exp.jpg){width=60%}</center>


Some of these __community detection__ methods can be applied
to scRNA-seq data by building a graph where each vertice represents a cell 
and (weight of) the edge measures similarity between two cells.  
Actually, graph-based clustering is the most popular clustering algorithm in 
scRNA-seq data analysis, and has been reported to have outperformed other
clustering methods in many situations [@freytag2018comparison].


##### Why do we want to represent the data as a graph?\
-  __Memory effectiveness__: A (complete) graph can be thought as an alternative expression of similarity matrix. Current methods (discuss later) aim to build sparse graphs, which ease the memory burden. \
- __Curse of dimensionality__: All data become sparse in high-dimensional space and therefore similarities measured by Euclidean distances etc are generally low between all objects.  \

##### Building a graph \
- __Step1__: Build an unweighted K-nearest neighbour (__KNN__) graph

<center>![](figures/NearestNeighborNetworks.png){width=40%}</center>


- __Step2__: Add weights, and obtain a shared nearest neighbour (__SNN__) graph

<center>![](figures/SNN.jpg){width=40%}</center>


There are two ways of adding weights: number and rank.\
- _number_: The number of shared nodes between $u$ and $v$, in this case, 3.    \
- _rank_: A measurement of the closeness to their common nearest neighbours. (@xu2015identification) \


 <font color="#bf812d">   
__Details of rank __: \
Main idea: The closeness of two people is defined by their closest common friend. \
For each node, say $u$, we can rank its 5 neighbours according to their closeness to $u$, and we can do the same with $v$.
Denote the three shared neighbours as $x_1$, $x_2$ and $x_3$, so rank($x_1, u$) = 1 means $x_1$ is the closest neighbour of $u$.
The idea is, if $x_1$ is also the closest to $v$, then $u$ and $v$ should have a larger similarity, or weight. 
So we summarize the overall closeness of $x_1$ with both $u$ and $v$ by taking an average: $\dfrac{1}{2}(\text{rank}(x_1, u), \text{rank}(x_1, v))$. 
Then we find the one with the largest closeness, $s(u, v) = \min \left[ \dfrac{1}{2}(\text{rank}(x_i, u), \text{rank}(x_i, v)) \vert i = 1, 2, 3\right]$.
The final expression of weight: 
$$ w(u, v) = K -  s(u, v).$$
</font>

 
#####  Quality function (Modularity)\
Modularity [@newman2004finding] is not the only quality function for graph-based clustering, 
but it is one of the first attempts to embed in a compact form many questions including
the definition of quality function and null model etc.\

__The idea of modularity__: A random graph should not have a cluster structure. \
The more "quality" a partition has compared to a random graph, the "better" the partition is.\
Specifically, it is defined by:

 the  <font color="#bf812d">  quality </font> of a partition on the actual graph $-$ the quality of the same partition on a   <font color="#bf812d"> random graph </font>
  
  <font color="#bf812d"> quality </font>: Sum of the weights within clusters \
   <font color="#bf812d"> random graph </font>: a copy of the original graph, with some of its properties, but without community structure.   The random graph defined by modularity is: each node has the same degree as the original graph. 
     
  $$ Q \propto \sum_{i, j} A_{i, j} \delta(i, j) - \sum_{i, j} \dfrac{k_i k_j}{2m} \delta(i, j)$$ 

- $A_{i, j}$: weight between node $i$ and $j$; 

- $\delta(i, j)$: indicator of whether $i$ and $j$ are in the same cluster; 

- $k_i$: the degree of node $i$ (the sum of weights of all edges connected to $i$);

- $m$: the total weight in the all graph.




__Higher modularity implies better partition__:
<center>![](figures/modularity.jpg){width=80%}</center>


__Limits of modularity__: [@good2010performance]\
1. Resolution limit. \
   Short version: Modularity maximization forces small communities into larger ones. \
   Longer version: For two clusters $A$ and $B$, if $k_A k_B < 2m$ then modularity increases by merging A and B into a single cluster, even if A and B are distinct clusters.\
2. Bad, even random partitions may have a high modularity.
<center>![](figures/modularity_limit1.jpg){width=80%}</center>

3. Networks lack a clear modularity maxima.

<center>![](figures/modularity_limit2.jpg){width=80%}</center>


#####  __Algorithms __:  \
Modularity-based clustering methods implemented in single cell analysis are mostly greedy algorithms, 
that are very fast, although not the most accurate approaches. 

   &nbsp;  &nbsp;  __Louvain__: [@blondel2008fast] 
   
<center>![](figures/Louvain.jpg){width=80%}</center>

    
   &nbsp;   &nbsp;  __Leiden__:[@traag2019louvain] \
   Improved Louvain, hybrid of greedy algorithm and sampling technique \
   
##### __Advantages__: \
  -Fast \
  -No need to specify $k$
   
##### __Tools for graph-based clustering__:\
   [__Seurat__](https://satijalab.org/seurat/): Louvain, Leiden, SLM \
   [__igraph__](https://igraph.org/): fast greedy, Louvain, optimal, walktrap, spinglass, infomap
   
  

####  Consensus clustering (more robustness, less computational speed)

##### __Motivation (Two problems of $K$-means)__: \
- __Problem1:__  sensitive to initial partitions \
 &nbsp; __Solution:__ 
  &nbsp; Run multiple iterations of $K$-means on different subsamples of the original dataset, with different initail partitions. \
- __Problem2:__   the selection of $K$.\
 &nbsp; __Solution:__ 
  &nbsp; Run $K$-means with a range of $K$'s.

##### __Algorithm of consensus clustering (simpliest version)__: 

```text
for(k in the range of K){
  for(each subsample of the data){
    for(iteration in 1:1000){
      kmeans(subsample, k) # each iteration means a different initial partition
      save partition
    }
  }
  return consensus clustering result of k
}
```

##### __Subsample obtained by dimensional reduction:__

steps of PCA: \
    i) tranformation of the similarity matrix.\
    ii) ranking eigen vectors according to their accoring eigen values in decreasing order.\
    iii) need to decide how many ($d$) PC's or eigenvalues we wants to reduce to.
    
In SC3, \
i) considers two types of transformation: 
the one with traditional PCA and the associated graph Laplacian. \
iii) User may specify a range of $d$, or use the default range suggested by the authors according to their experience with empirical results.


##### __Consensus clustering (combining multiple clustering results):__ \

- __Step1:__ Represent each partition as a matrix: \
Say we partitioned four data points into 2 clusters.

<center>![](figures/concensus1.png){width=60%}</center>

- __Step2:__ Consensus matrix: \
 Average of all the partitions
 <center>![](figures/concensus2.png){width=30%}</center>



##### __Tools for consensus clustering__: [SC3](http://bioconductor.org/packages/SC3/)


![](figures/sc3.png)



