---
output: html_document
---





## Clustering example {#clust-methods}


```r
library(pcaMethods)
library(SC3)
library(scater)
library(SingleCellExperiment)
library(pheatmap)
library(mclust)
library(igraph)
library(scran)
```


### Example 1. Graph-based clustering (deng dataset)

To illustrate clustering of scRNA-seq data, we consider the `Deng` dataset of
cells from developing mouse embryo [@Deng2014-mx]. We have preprocessed the
dataset and created a `SingleCellExperiment` object in advance. We have also
annotated the cells with the cell types identified in the original publication
(it is the `cell_type2` column in the `colData` slot).


```r
deng <- readRDS("data/deng/deng-reads.rds")
```

First, we build a $K$-NN graph with a package function from
[scran](https://bioconductor.org/packages/release/bioc/html/scran.html). The
most important decision of building a graph is the choice of $K$, of which there
is no standard rule. In general, we can think of it as an indication of the
desired cluster size. If $K$ is too small, a genuine cluster might be split into
parts, while if $K$ is too large, clusters might not thoroughly separated.


```r
deng5 <- buildSNNGraph(deng, k = 5)
deng15 <- buildSNNGraph(deng, k = 15)
deng25 <- buildSNNGraph(deng, k = 25)
par(mfrow=c(1,3))
plot(deng5, vertex.size = 4, vertex.label = NA)
title("5-NN" ,line = -33, cex.main = 3)
plot(deng15, vertex.size = 4, vertex.label = NA)
title("15-NN" ,line = -33, cex.main = 3)
plot(deng25, vertex.size = 4, vertex.label = NA)
title("25-NN" ,line = -33, cex.main = 3)
```

<img src="clustering_files/figure-html/unnamed-chunk-2-1.png" width="1152" style="display: block; margin: auto;" />

Perform Louvain clustering: 

```r
cl <- igraph::cluster_louvain(deng15)$membership
colData(deng)$cl <- factor(cl)
mclust::adjustedRandIndex(colData(deng)$cell_type2, colData(deng)$cl)
```

```
## [1] 0.4197754
```
Reaches very high similarity with the labels provided in the original paper. 

However, it tend to merge small clusters into larger ones. 


```r
table(deng$cell_type2, cl)
```

```
##             cl
##               1  2  3
##   16cell     49  0  1
##   4cell       0 14  0
##   8cell      36  0  1
##   early2cell  0  8  0
##   earlyblast  0  0 43
##   late2cell   0 10  0
##   lateblast   0  0 30
##   mid2cell    0 12  0
##   midblast    0  0 60
##   zy          0  4  0
```



### Example 2. Graph-based clustering (segerstolpe dataset)


```r
muraro <- readRDS("data/pancreas/muraro.rds")
## PCA
var.fit <- suppressWarnings(trendVar(muraro, parametric=TRUE, use.spikes=F))
muraro <- suppressWarnings(denoisePCA(muraro, technical=var.fit$trend))
dim(reducedDim(muraro, "PCA"))
```

```
## [1] 2126    5
```

```r
## Build graph and clustering
gr <- buildSNNGraph(muraro, use.dimred="PCA", k = 30)
cl <- igraph::cluster_louvain(gr)$membership
colData(muraro)$cl <- factor(cl)
mclust::adjustedRandIndex(colData(muraro)$cell_type1, colData(muraro)$cl)
```

```
## [1] 0.4845618
```


```r
table(muraro$cell_type1, cl)
```

```
##              cl
##                 1   2   3   4   5   6   7   8   9
##   acinar        0   0   0   0   0   0 218   0   1
##   alpha       202 306 274   5  15   9   1   0   0
##   beta          1   0   0   5 195  21   2 220   4
##   delta         0   0   0   0  18 174   0   1   0
##   ductal        0   0   0 215   0   1   7   3  19
##   endothelial   0   0   0   0   0   0   0   0  21
##   epsilon       0   0   0   0   0   3   0   0   0
##   gamma         1   0   1   0   0  97   2   0   0
##   mesenchymal   0   0   0   1   0   0   0   0  79
##   unclear       0   0   0   4   0   0   0   0   0
```

### Example 3. SC3

Let's run `SC3` clustering on the Deng data. The advantage of the `SC3` is that it can directly ingest a `SingleCellExperiment` object.

`SC3` can estimate a number of clusters:

```r
deng <- sc3_estimate_k(deng)
```

```
## Estimating k...
```

```r
metadata(deng)$sc3$k_estimation
```

```
## [1] 6
```


Next we  run `SC3` (we also ask it to calculate biological properties of the clusters): 

```r
deng <- sc3(deng, ks = 10, biology = TRUE, n_cores = 1)
```

```
## Setting SC3 parameters...
```

```
## Calculating distances between the cells...
```

```
## Performing transformations and calculating eigenvectors...
```

```
## Performing k-means clustering...
```

```
## Calculating consensus matrix...
```

```
## Calculating biology...
```

`SC3` result consists of several different outputs (please look in [@Kiselev2016-bq] and [SC3 vignette](http://bioconductor.org/packages/release/bioc/vignettes/SC3/inst/doc/my-vignette.html) for more details). Here we show some of them:

Consensus matrix:

```r
sc3_plot_consensus(deng, k = 10, show_pdata = "cell_type2")
```

Silhouette plot:

```r
sc3_plot_silhouette(deng, k = 10)
```

<img src="clustering_files/figure-html/unnamed-chunk-7-1.png" width="672" style="display: block; margin: auto;" />

Heatmap of the expression matrix:

```r
sc3_plot_expression(deng, k = 10, show_pdata = "cell_type2")
```

<img src="clustering_files/figure-html/unnamed-chunk-8-1.png" width="672" style="display: block; margin: auto;" />

Identified marker genes:

```r
sc3_plot_markers(deng, k = 10, show_pdata = "cell_type2")
```

<img src="clustering_files/figure-html/unnamed-chunk-9-1.png" width="672" style="display: block; margin: auto;" />

PCA plot with highlighted `SC3` clusters:

```r
plotPCA(deng, colour_by = "sc3_10_clusters")
```

<img src="clustering_files/figure-html/unnamed-chunk-10-1.png" width="672" style="display: block; margin: auto;" />

Compare the results of `SC3` clustering with the original publication cell type labels:

```r
adjustedRandIndex(colData(deng)$cell_type2, colData(deng)$sc3_10_clusters)
```

```
## [1] 0.7804189
```

__Note__ `SC3` can also be run in an interactive `Shiny` session:

```r
sc3_interactive(deng)
```

This command will open `SC3` in a web browser.

__Note__ Due to direct calculation of distances `SC3` becomes very slow when the number of cells is $>5000$. For large datasets containing up to $10^5$ cells we recomment using `Seurat` (see chapter \@ref(seurat-chapter)).

## An alternative to clustering: Automatic cell annotation 

### [SingleR]("https://bioconductor.org/packages/devel/bioc/html/SingleR.html")

#### Methodology 

 - __Step1. Find variable gene__ \
    __1-1.__ For every gene, obtain __median__ grouped by label.
    <center> ![](figures/SingleR_median.png){width=80%} </center>
    __1-2.__ Select genes that makes at least one label different:. 
    For example, if we are looking for the genes that makes label "green" different from label "red", 
    we substract the second column by the first, and pick the top $N$ highest and positive values. 
    <center> ![](figures/SingleR_VarGene.png){width=80%} </center>
    All analysis onwards use only the selected variable genes. \
 - __Step2. Spearman's correlation__ \
  Spearman's correlation $\in [-1, 1]$ is a  measure of the strength of a linear or monotonic relationship between paired data. 
  <center> ![](figures/spearman.png){width=80%} </center>
  We compute the Spearman's correlation for all pairs of cells in the test and reference dataset, and 
  obtain an $n_{\text{test}} \times n_{\text{ref}}$ correlation matrix, where $n$ is the number of cells
  (see the first matrix in Step3). \
  - __Step3. Scoring__ \
  We want to know how each cell in the test data is correlated to the labels in the reference data, instead of each reference cell. So we take the correlations of a cell in the test data with all the cells with a certain label in the reference data, and summarize them into one number or a score, in SingleR, the default is to take the $80\%$ quantile. 
  <center> ![](figures/SingleR_score.png){width=80%} </center>
 - __Step4. Fine tuning__\
  We stop here and assign each cell with label that score the highest, actually, if we set the argument ```fine.tune = FALSE```, that is exactly what the package function ```SingleR``` does. 
  But there is one more question, what if the second highest score is very close to the highest? say, 1, 1, 1, 9.5, 10.
  `SingleR` set a threshold to define how close is "very close", the default is 0.05. 
  For (only) the cells that falls into this category, it goes back to Step2.


#### Example

(Note: SingleR is not yet available in the released version of Bioconductor. It will be possible to run it as shown once the next Bioconductor release is made in late October.)


```r
library(scRNAseq)
library(SingleR)
segerstolpe <- readRDS("data/pancreas/segerstolpe.rds")
sceM <- suppressMessages(MuraroPancreasData())
sceM <- sceM[,!is.na(sceM$label)]
sceM <- logNormCounts(sceM)
## find common gene
rownames(sceM) <- gsub("__.*","",rownames(sceM))
common <- intersect(rownames(sceM), rownames(segerstolpe))
sceM <- sceM[common,]
segerstolpe <- segerstolpe[common,]
## Prepare reference
out <- pairwiseTTests(logcounts(sceM), sceM$label, direction="up")
markers <- getTopMarkers(out$statistics, out$pairs, n=10)
## Annotation
pred <- SingleR(test=segerstolpe, ref=sceM, labels=sceM$label, genes=markers)
## View result 
plotScoreHeatmap(pred, show.labels = TRUE,  annotation_col=data.frame(
  row.names=rownames(pred)))
```

<center>![](figures/SingleR.png){width=80%} </center>

### [scmap](https://bioconductor.org/packages/release/bioc/html/scmap.html)


```r
## Load data
segerstolpe <- readRDS("data/pancreas/segerstolpe.rds")  # test
library(scRNAseq)
sceM <- readRDS("data/pancreas/muraro.rds") # reference
rownames(sceM) <- gsub("__.*","",rownames(sceM))
```

Select the most informative features (genes) using the dropout feature selection method. 
By default select 500 features. 


```r
library(scmap)
rowData(sceM)$feature_symbol <- rownames(sceM)
sceM <- selectFeatures(sceM, suppress_plot = TRUE)
```


Index of a reference dataset is created by finding the median gene expression for each cluster. 
First, chop the total of 500 features into $M = 50$ chuncks/ low-dimensional subspace. 
Second, cluster each chunk into $k = \sqrt{N}$ clusters, where $N$ is the number of cells.

By default scmap uses the `cell_type1` column of the `colData` slot in the
reference to identify clusters.


```r
sceM <- indexCell(sceM)
```

The function ```indexCluster``` writes the ```scmap_cluster_index``` item of the meta data slot of the reference dataset ```sceM```.
This step has two outputs:


```r
names(metadata(sceM)$scmap_cell_index)
```

```
## [1] "subcentroids" "subclusters"
```

1. ```subcentroids``` returns cluster centers: 

```r
cat(length(metadata(sceM)$scmap_cell_index$subcentroids), " chunks \n")
```

```
## 50  chunks
```

```r
cat("The dimension of cluster centers in each chunk: ", dim(metadata(sceM)$scmap_cell_index$subcentroids[[1]]), "\n")
```

```
## The dimension of cluster centers in each chunk:  10 46
```

2. ```subclusters``` contains information about which cluster (label) the cells belong to

```r
dim(metadata(sceM)$scmap_cell_index$subclusters)
```

```
## [1]   50 2126
```

```r
metadata(sceM)$scmap_cell_index$subclusters[1:5,1:5]
```

```
##      D28.1_1 D28.1_13 D28.1_15 D28.1_17 D28.1_2
## [1,]       8       14        5        3       3
## [2,]      15       40        8       35       4
## [3,]       7       43       21        1      20
## [4,]      27       42       34       27      30
## [5,]      45       38        4       41      36
```


__Projection:__ Once the scmap-cell indexes have been generated we can use them to project the test dataset.

```r
scmapCell_results <- scmapCell(
  projection = segerstolpe,
  index_list = list(
    sceM = metadata(sceM)$scmap_cell_index
  ) )
names(scmapCell_results)
```

```
## [1] "sceM"
```

The ```cells``` matrix contains the top 10 (scmap default) cell IDs of the cells
of the reference dataset that a given cell of the projection dataset is closest
to:


```r
dim(scmapCell_results$sceM$cells)
```

```
## [1]   10 3514
```

__Cell annotation:__ If cell cluster annotation is available for the reference
datasets, scmap-cell can also annotate the cells from the projection dataset
using the labels of the reference. It does so by looking at the top 3 nearest
neighbours (scmap default) and if they all belong to the same cluster in the
reference and their maximum similarity is higher than a threshold (0.5 is the
scmap default), then a projection cell is assigned to the corresponding
reference cluster:


```r
scmapCell_clusters <- scmapCell2Cluster(
  scmapCell_results,
  list(
    colData(sceM)$cell_type1
  ))
```


__Plot result __ Compare the annotated result with the original label in the
`segerstolpe` dataset.


```r
plot(
  getSankey(
    segerstolpe$cell_type1,
    scmapCell_clusters$combined_labs, plot_height = 400 )
)
```

<center>![](figures/sankey.png){width=80%}</center>


### sessionInfo()




