---
knit: bookdown::preview_chapter
---




## Identifying confounding factors

### Introduction

There is a large number of potential confounders, artifacts and biases in
scRNA-seq data. One of the main challenges in analysing scRNA-seq data stems
from the fact that it is difficult to carry out a true technical replication
(why?) to distinguish biological and technical variability. In the previous
chapters we considered normalization and in this chapter we will continue to
explore how experimental artifacts can be identified and removed. We will
continue using the `scater` package since it provides a set of methods
specifically for quality control of experimental and explanatory variables.
Moreover, we will continue to work with the Blischak data that was used in the
previous chapter.



```r
library(scater, quietly = TRUE)
library(scran)
options(stringsAsFactors = FALSE)
umi <- readRDS("data/tung/umi.rds")
umi.qc <- umi[rowData(umi)$use, colData(umi)$use]
endog_genes <- !rowData(umi.qc)$is_feature_control
```

The `umi.qc` dataset contains filtered cells and genes. Our next step is to
explore technical drivers of variability in the data to inform data
normalisation before downstream analysis.

### Correlations with PCs

Let's first look again at the PCA plot of the QCed dataset using the
scran-normalized log2-CPM values:


```r
qclust <- quickCluster(umi.qc, min.size = 30, use.ranks = FALSE)
umi.qc <- computeSumFactors(umi.qc, sizes = 15, clusters = qclust)
umi.qc <- normalize(umi.qc)
reducedDim(umi.qc, "PCA") <- reducedDim(
  runPCA(umi.qc[endog_genes,],
         exprs_values = "logcounts", ncomponents = 10), "PCA")
plotPCA(
    umi.qc,
    colour_by = "batch",
    size_by = "total_features_by_counts"
)
```

<div class="figure" style="text-align: center">
<img src="confounders_files/figure-html/confound-pca-1.png" alt="PCA plot of the tung data" width="90%" />
<p class="caption">(\#fig:confound-pca)PCA plot of the tung data</p>
</div>

`scater` allows one to identify principal components that correlate with
experimental and QC variables of interest (it ranks principle components by
$R^2$ from a linear model regressing PC value against the variable of interest).

Let's test whether some of the variables correlate with any of the PCs.

#### Top colData variables associated with PCs

The plot below shows, for each of the first 10 PCs, the variance explained by
the ten variables in `colData(umi.qc)` that are most strongly associated with
the PCs. [We will ignore the `sample_id` variable: it has a unique value for
each cell, so can explain all the variation for all PCs.]


```r
plotExplanatoryPCs(umi.qc)
```

<div class="figure" style="text-align: center">
<img src="confounders_files/figure-html/confound-find-pcs-total-features-1.png" alt="PC correlation with the number of detected genes" width="90%" />
<p class="caption">(\#fig:confound-find-pcs-total-features)PC correlation with the number of detected genes</p>
</div>

Indeed, we can see that `PC1` can be almost completely explained by `batch` and
`individual` (of course batch is nested within individual). The total counts
from ERCC spike-ins also explains a substantial proportion of the variability in
PC1.

Although number of detected genes is not strongly correlated with the PCs here
(after normalization), this is commonly the case and something to look out for.
[You might like to replicate the plot above using raw logcounts values to see
what happens without normalization]. This is a well-known issue in scRNA-seq and
was described [here](http://biorxiv.org/content/early/2015/12/27/025528).

### Explanatory variables

`scater` can also compute the marginal $R^2$ for each variable when fitting a
linear model regressing expression values for each gene against just that
variable, and display a density plot of the gene-wise marginal $R^2$ values for
the variables.


```r
plotExplanatoryVariables(
    umi.qc,
    exprs_values = "logcounts_raw",
    variables = c(
        "total_features_by_counts",
        "total_counts",
        "batch",
        "individual",
        "pct_counts_ERCC",
        "pct_counts_MT"
    )
)
```

<div class="figure" style="text-align: center">
<img src="confounders_files/figure-html/confound-find-expl-vars-1.png" alt="Explanatory variables" width="90%" />
<p class="caption">(\#fig:confound-find-expl-vars)Explanatory variables</p>
</div>

This analysis indicates that the number of detected genes (again) and also the
sequencing depth (total number of UMI counts per cell) have substantial
explanatory power for many genes, so these variables are good candidates for
conditioning out in a normalization step, or including in downstream statistical
models [cf. `sctransform`'s approach to normalization]. Expression of ERCCs also
appears to be an important explanatory variable and one notable feature of the
above plot is that batch explains more than individual. What does that tell us
about the technical and biological variability of the data?

### Other confounders

In addition to correcting for batch, there are other factors that one may want
to compensate for. As with batch correction, these adjustments require extrinsic
information. One popular method is [scLVM](https://github.com/PMBio/scLVM) which
allows you to identify and subtract the effect from processes such as cell-cycle
or apoptosis.

In addition, protocols may differ in terms of their coverage of each transcript,
their bias based on the average content of __A/T__ nucleotides, or their ability
to capture short transcripts. Ideally, we would like to compensate for all of
these differences and biases.

### Exercise

Perform the same analysis with read counts of the Blischak data. Use
`tung/reads.rds` file to load the reads SCESet object. Once you have finished
please compare your results to ours (next chapter).

### sessionInfo()


```
## R version 3.6.0 (2019-04-26)
## Platform: x86_64-pc-linux-gnu (64-bit)
## Running under: Ubuntu 18.04.3 LTS
## 
## Matrix products: default
## BLAS:   /usr/lib/x86_64-linux-gnu/blas/libblas.so.3.7.1
## LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.7.1
## 
## locale:
##  [1] LC_CTYPE=en_AU.UTF-8       LC_NUMERIC=C              
##  [3] LC_TIME=en_AU.UTF-8        LC_COLLATE=en_AU.UTF-8    
##  [5] LC_MONETARY=en_AU.UTF-8    LC_MESSAGES=en_AU.UTF-8   
##  [7] LC_PAPER=en_AU.UTF-8       LC_NAME=C                 
##  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
## [11] LC_MEASUREMENT=en_AU.UTF-8 LC_IDENTIFICATION=C       
## 
## attached base packages:
## [1] parallel  stats4    stats     graphics  grDevices utils     datasets 
## [8] methods   base     
## 
## other attached packages:
##  [1] scran_1.12.1                scater_1.12.2              
##  [3] ggplot2_3.2.1               SingleCellExperiment_1.6.0 
##  [5] SummarizedExperiment_1.14.1 DelayedArray_0.10.0        
##  [7] BiocParallel_1.18.1         matrixStats_0.55.0         
##  [9] Biobase_2.44.0              GenomicRanges_1.36.1       
## [11] GenomeInfoDb_1.20.0         IRanges_2.18.3             
## [13] S4Vectors_0.22.1            BiocGenerics_0.30.0        
## [15] knitr_1.25                 
## 
## loaded via a namespace (and not attached):
##  [1] locfit_1.5-9.1           Rcpp_1.0.2              
##  [3] rsvd_1.0.2               lattice_0.20-38         
##  [5] assertthat_0.2.1         digest_0.6.21           
##  [7] R6_2.4.0                 dynamicTreeCut_1.63-1   
##  [9] evaluate_0.14            highr_0.8               
## [11] pillar_1.4.2             zlibbioc_1.30.0         
## [13] rlang_0.4.0              lazyeval_0.2.2          
## [15] irlba_2.3.3              Matrix_1.2-17           
## [17] rmarkdown_1.15           labeling_0.3            
## [19] BiocNeighbors_1.2.0      statmod_1.4.32          
## [21] stringr_1.4.0            igraph_1.2.4.1          
## [23] RCurl_1.95-4.12          munsell_0.5.0           
## [25] compiler_3.6.0           vipor_0.4.5             
## [27] BiocSingular_1.0.0       xfun_0.9                
## [29] pkgconfig_2.0.3          ggbeeswarm_0.6.0        
## [31] htmltools_0.3.6          tidyselect_0.2.5        
## [33] tibble_2.1.3             gridExtra_2.3           
## [35] GenomeInfoDbData_1.2.1   bookdown_0.13           
## [37] edgeR_3.26.8             viridisLite_0.3.0       
## [39] crayon_1.3.4             dplyr_0.8.3             
## [41] withr_2.1.2              bitops_1.0-6            
## [43] grid_3.6.0               gtable_0.3.0            
## [45] magrittr_1.5             scales_1.0.0            
## [47] dqrng_0.2.1              stringi_1.4.3           
## [49] XVector_0.24.0           viridis_0.5.1           
## [51] limma_3.40.6             DelayedMatrixStats_1.6.1
## [53] cowplot_1.0.0            tools_3.6.0             
## [55] glue_1.3.1               beeswarm_0.2.3          
## [57] purrr_0.3.2              yaml_2.2.0              
## [59] colorspace_1.4-1
```
