---
output: html_document
---




## Data visualization and exploratory data analysis

### Introduction

In this chapter we will continue to work with the filtered `Tung` dataset
produced in the previous chapter. We will explore different ways of visualizing
the data to allow you to asses what happened to the expression matrix after the
quality control step. `scater` package provides several very useful functions to
simplify visualisation.

One important aspect of single-cell RNA-seq is to control for batch effects.
Batch effects are technical artefacts that are added to the samples during
handling. For example, if two sets of samples were prepared in different labs or
even on different days in the same lab, then we may observe greater similarities
between the samples that were handled together. In the worst case scenario,
batch effects may be [mistaken](http://f1000research.com/articles/4-121/v1) for
true biological variation.  

Data visualisation can help to identify batch effects or other unwanted sources
of variation that affect our observed gene expression measurements. The `Tung`
data allows us to explore these issues in a controlled manner since some of the
salient aspects of how the samples were handled have been recorded. Ideally, we
expect to see batches from the same individual grouping together and distinct
groups corresponding to each individual.

Data visualisation and exploratory data analysis are invaluable for allowing us
to get a "feel" for a dataset. This is an area of data analysis that is perhaps
[more art than science](https://www.youtube.com/watch?v=W5JqB6e5QwU), but is a
crucial aspect of single-cell QC and analysis.



```r
library(SingleCellExperiment)
library(scater)
options(stringsAsFactors = FALSE)
umi <- readRDS("data/tung/umi.rds")
umi.qc <- umi[rowData(umi)$use, colData(umi)$use]
endog_genes <- !rowData(umi.qc)$is_feature_control
```

### PCA plot {#visual-pca}

The easiest way to overview the data is by transforming it using the principal
component analysis and then visualize the first two principal components.

[Principal component analysis
(PCA)](https://en.wikipedia.org/wiki/Principal_component_analysis) is a
statistical procedure that uses a transformation to convert a set of
observations into a set of values of linearly uncorrelated variables called
principal components (PCs). The number of principal components is less than or
equal to the number of original variables.

Mathematically, the PCs correspond to the eigenvectors of the covariance matrix.
The eigenvectors are sorted by eigenvalue so that the first principal component
accounts for as much of the variability in the data as possible, and each
succeeding component in turn has the highest variance possible under the
constraint that it is orthogonal to the preceding components (the figure below
is taken from
[here](http://www.nlpca.org/pca_principal_component_analysis.html)).

<div class="figure" style="text-align: center">
<img src="figures/pca.png" alt="Schematic representation of PCA dimensionality reduction" width="100%" />
<p class="caption">(\#fig:clust-pca)Schematic representation of PCA dimensionality reduction</p>
</div>

#### Before QC

Without log-transformation:

```r
tmp <- runPCA(
  umi[endog_genes, ],
  exprs_values = "counts"
)
plotPCA(
    tmp,
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
)
```

<div class="figure" style="text-align: center">
<img src="data-viz_files/figure-html/expr-overview-pca-before-qc1-1.png" alt="PCA plot of the tung data" width="90%" />
<p class="caption">(\#fig:expr-overview-pca-before-qc1)PCA plot of the tung data</p>
</div>

With log-transformation:

```r
tmp <- runPCA(
  umi[endog_genes, ],
  exprs_values = "logcounts_raw"
)
plotPCA(
    tmp,
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
)
```

<div class="figure" style="text-align: center">
<img src="data-viz_files/figure-html/expr-overview-pca-before-qc2-1.png" alt="PCA plot of the tung data" width="90%" />
<p class="caption">(\#fig:expr-overview-pca-before-qc2)PCA plot of the tung data</p>
</div>

Clearly log-transformation is benefitial for our data - it reduces the variance
on the first principal component and already separates some biological effects.
Moreover, it makes the distribution of the expression values more normal. In the
following analysis and chapters we will be using log-transformed raw counts by
default.

__However, note that just a log-transformation is not enough to account for
different technical factors between the cells (e.g. sequencing depth).
Therefore, please do not use `logcounts_raw` for your downstream analysis,
instead as a minimum suitable data use the `logcounts` slot of the
`SingleCellExperiment` object, which not just log-transformed, but also
normalised by library size (e.g. CPM normalisation). In the course we use
`logcounts_raw` only for demonstration purposes!__

#### After QC


```r
tmp <- runPCA(
  umi.qc[endog_genes, ],
  exprs_values = "logcounts_raw"
)
plotPCA(
    tmp,
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
)
```

<div class="figure" style="text-align: center">
<img src="data-viz_files/figure-html/expr-overview-pca-after-qc-1.png" alt="PCA plot of the tung data" width="90%" />
<p class="caption">(\#fig:expr-overview-pca-after-qc)PCA plot of the tung data</p>
</div>

Comparing Figure \@ref(fig:expr-overview-pca-before-qc2) and Figure
\@ref(fig:expr-overview-pca-after-qc), it is clear that after quality control
the NA19098.r2 cells no longer form a group of outliers.

By default only the top 500 most variable genes are used by scater to calculate
the PCA. This can be adjusted by changing the `ntop` argument.

__Exercise 1__

How do the PCA plots change if when all 14,066 genes are used? Or when only top
50 genes are used? Why does the fraction of variance accounted for by the first
PC change so dramatically?

__Hint__ Use `ntop` argument of the `plotPCA` function.

__Our answer__

<div class="figure" style="text-align: center">
<img src="data-viz_files/figure-html/expr-overview-pca-after-qc-exercise1-1-1.png" alt="PCA plot of the tung data (14214 genes)" width="90%" />
<p class="caption">(\#fig:expr-overview-pca-after-qc-exercise1-1)PCA plot of the tung data (14214 genes)</p>
</div>

<div class="figure" style="text-align: center">
<img src="data-viz_files/figure-html/expr-overview-pca-after-qc-exercise1-2-1.png" alt="PCA plot of the tung data (50 genes)" width="90%" />
<p class="caption">(\#fig:expr-overview-pca-after-qc-exercise1-2)PCA plot of the tung data (50 genes)</p>
</div>

If your answers are different please compare your code with
[ours](https://github.com/hemberg-lab/scRNA.seq.course/blob/master/07-exprs-overview.Rmd)
(you need to search for this exercise in the opened file).

### tSNE map {#visual-tsne}

An alternative to PCA for visualizing scRNA-seq data is a tSNE plot.
[tSNE](https://lvdmaaten.github.io/tsne/) (t-Distributed Stochastic Neighbor
Embedding) converts high-dimensional Euclidean distances between datapoints into
conditional probabilities that represent similarities, to produce a
low-dimensional representation of high-dimensional data that displays large- and
local-scale structure in the dataset. Here, we map high dimensional data (i.e.
our 14,214 dimensional expression matrix) to a 2-dimensional space while
preserving local distances between cells. tSNE is almost always used to produce
a two-dimensional representation of a high-dimensional dataset; it is only
rarely used to generate a reduced-dimension space with more than two dimensions
and is typically used only for visulisation as opposed being used as a general
dimension-reduction method.

Due to the non-linear and stochastic nature of the algorithm, tSNE is more
difficult to intuitively interpret than a standard dimensionality reduction
method such as PCA. Things to be aware of when using tSNE:

* tSNE has a tendency to (visually) cluster points; as such, it often creates
  attractive plots of datasets with distinct cell types, but does look as good
  when there are continuous changes in the cell population.
* The hyperparameters really matter: in particular, changing the _perplexity_
  parameter can have a large effect on the visulisation produced. Perplexity is
  a measure of information, but can loosely be thought of as a tuning parameter
  that controls the number of _nearest neighbous_ for each datapoint.
* Cluster sizes in a tSNE plot mean nothing.
* Distances between clusters might not mean anything.
* Random noise doesn't always look random.
* You can see some shapes, sometimes.

For more details about how to use tSNE effectively, see [this exellent
article](https://distill.pub/2016/misread-tsne/).

In contrast with PCA, tSNE is a stochastic algorithm which means running the
method multiple times on the same dataset will result in different plots. To
ensure reproducibility, we fix the "seed" of the random-number generator in the
code below so that we always get the same plot.


#### Before QC


```r
set.seed(123456)
tmp <- runTSNE(
    umi[endog_genes, ],
    exprs_values = "logcounts_raw",
    perplexity = 130
)
plotTSNE(
    tmp,
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
)
```

<div class="figure" style="text-align: center">
<img src="data-viz_files/figure-html/expr-overview-tsne-before-qc-1.png" alt="tSNE map of the tung data" width="90%" />
<p class="caption">(\#fig:expr-overview-tsne-before-qc)tSNE map of the tung data</p>
</div>

#### After QC


```r
set.seed(123456)
tmp <- runTSNE(
    umi.qc[endog_genes, ],
    exprs_values = "logcounts_raw",
    perplexity = 130
)
plotTSNE(
    tmp,
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
)
```

<div class="figure" style="text-align: center">
<img src="data-viz_files/figure-html/expr-overview-tsne-after-qc-1.png" alt="tSNE map of the tung data" width="90%" />
<p class="caption">(\#fig:expr-overview-tsne-after-qc)tSNE map of the tung data</p>
</div>

Interpreting PCA and tSNE plots is often challenging and due to their stochastic
and non-linear nature, they are less intuitive. However, in this case it is
clear that they provide a similar picture of the data. Comparing Figure
\@ref(fig:expr-overview-tsne-before-qc) and
\@ref(fig:expr-overview-tsne-after-qc), it is again clear that the samples from
NA19098.r2 are no longer outliers after the QC filtering.

Furthermore tSNE requires you to provide a value of `perplexity` which reflects
the number of neighbours used to build the nearest-neighbour network; a high
value creates a dense network which clumps cells together while a low value
makes the network more sparse allowing groups of cells to separate from each
other. `scater` uses a default perplexity of the total number of cells divided
by five (rounded down).

You can read more about the pitfalls of using tSNE [here](http://distill.pub/2016/misread-tsne/).

UMAP (Uniform Manifold Approximation and Projection) is a newer alternative to
tSNE which also often creates attractive visualisations of scRNA-seq data with
the benefit of being faster than tSNE to compute and is a "true" dimensionality
reduction method.

We will look at PCA, tSNE and UMAP plots in subsequent chapters and discuss the
topic of dimensionality reduction further in the [Latent spaces]{#latent-spaces}
chapter.

__Exercise 2__

How do the tSNE plots change when a perplexity of 10 or 200 is used? How does
the choice of perplexity affect the interpretation of the results?

__Our answer__

<div class="figure" style="text-align: center">
<img src="data-viz_files/figure-html/expr-overview-tsne-after-qc-exercise2-1-1.png" alt="tSNE map of the tung data (perplexity = 10)" width="90%" />
<p class="caption">(\#fig:expr-overview-tsne-after-qc-exercise2-1)tSNE map of the tung data (perplexity = 10)</p>
</div>

<div class="figure" style="text-align: center">
<img src="data-viz_files/figure-html/expr-overview-tsne-after-qc-exercise2-2-1.png" alt="tSNE map of the tung data (perplexity = 200)" width="90%" />
<p class="caption">(\#fig:expr-overview-tsne-after-qc-exercise2-2)tSNE map of the tung data (perplexity = 200)</p>
</div>

### Big Exercise

Perform the same analysis with read counts of the Blischak data. Use
`tung/reads.rds` file to load the reads SCE object. Once you have finished
please compare your results to ours (next chapter).

### sessionInfo()


```
## R version 3.6.0 (2019-04-26)
## Platform: x86_64-pc-linux-gnu (64-bit)
## Running under: Ubuntu 18.04.3 LTS
## 
## Matrix products: default
## BLAS:   /usr/lib/x86_64-linux-gnu/blas/libblas.so.3.7.1
## LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.7.1
## 
## locale:
##  [1] LC_CTYPE=en_AU.UTF-8       LC_NUMERIC=C              
##  [3] LC_TIME=en_AU.UTF-8        LC_COLLATE=en_AU.UTF-8    
##  [5] LC_MONETARY=en_AU.UTF-8    LC_MESSAGES=en_AU.UTF-8   
##  [7] LC_PAPER=en_AU.UTF-8       LC_NAME=C                 
##  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
## [11] LC_MEASUREMENT=en_AU.UTF-8 LC_IDENTIFICATION=C       
## 
## attached base packages:
## [1] parallel  stats4    stats     graphics  grDevices utils     datasets 
## [8] methods   base     
## 
## other attached packages:
##  [1] scater_1.12.2               ggplot2_3.2.1              
##  [3] SingleCellExperiment_1.6.0  SummarizedExperiment_1.14.1
##  [5] DelayedArray_0.10.0         BiocParallel_1.18.1        
##  [7] matrixStats_0.55.0          Biobase_2.44.0             
##  [9] GenomicRanges_1.36.1        GenomeInfoDb_1.20.0        
## [11] IRanges_2.18.3              S4Vectors_0.22.1           
## [13] BiocGenerics_0.30.0        
## 
## loaded via a namespace (and not attached):
##  [1] Rcpp_1.0.2               rsvd_1.0.2              
##  [3] lattice_0.20-38          assertthat_0.2.1        
##  [5] digest_0.6.21            R6_2.4.0                
##  [7] evaluate_0.14            highr_0.8               
##  [9] pillar_1.4.2             zlibbioc_1.30.0         
## [11] rlang_0.4.0              lazyeval_0.2.2          
## [13] irlba_2.3.3              Matrix_1.2-17           
## [15] rmarkdown_1.15           BiocNeighbors_1.2.0     
## [17] labeling_0.3             Rtsne_0.15              
## [19] stringr_1.4.0            RCurl_1.95-4.12         
## [21] munsell_0.5.0            compiler_3.6.0          
## [23] vipor_0.4.5              BiocSingular_1.0.0      
## [25] xfun_0.9                 pkgconfig_2.0.3         
## [27] ggbeeswarm_0.6.0         htmltools_0.3.6         
## [29] tidyselect_0.2.5         tibble_2.1.3            
## [31] gridExtra_2.3            GenomeInfoDbData_1.2.1  
## [33] bookdown_0.13            viridisLite_0.3.0       
## [35] crayon_1.3.4             dplyr_0.8.3             
## [37] withr_2.1.2              bitops_1.0-6            
## [39] grid_3.6.0               gtable_0.3.0            
## [41] magrittr_1.5             scales_1.0.0            
## [43] stringi_1.4.3            XVector_0.24.0          
## [45] viridis_0.5.1            DelayedMatrixStats_1.6.1
## [47] cowplot_1.0.0            tools_3.6.0             
## [49] glue_1.3.1               beeswarm_0.2.3          
## [51] purrr_0.3.2              yaml_2.2.0              
## [53] colorspace_1.4-1         knitr_1.25
```
