---
output: html_document
---



# Differential Expression (DE) analysis {#dechapter}

## Introduction to DE analysis

### Bulk RNA-seq

One of the most common types of analyses when working with bulk RNA-seq data is
to identify differentially expressed genes. By comparing the genes that change
between two or more conditions, e.g. mutant and wild-type or stimulated and
unstimulated, it is possible to characterize the molecular mechanisms underlying
the change.

Several different methods, e.g.
[edgeR](https://bioconductor.org/packages/release/bioc/html/edgeR.html) and
[DESeq2](https://bioconductor.org/packages/DESeq2) and more, have been developed
for bulk RNA-seq and become established as parts of robust and widely-used
analysis workflows. Moreover, there are also extensive
[datasets](http://genomebiology.biomedcentral.com/articles/10.1186/gb-2013-14-9-r95)
available where the RNA-seq data has been validated using RT-qPCR. These data
can be used to benchmark DE finding algorithms and the available evidence
suggests that the algorithms are performing well.


### Single cell RNA-seq

In contrast to bulk RNA-seq, in scRNA-seq we often do not have a defined set
of experimental conditions. Instead, as was shown in a previous chapter
(\@ref(clust-methods)) we can identify the cell groups by using an unsupervised
clustering approach. Once the groups have been identified one can find
differentially expressed genes either by comparing the differences in variance
between the groups (like the Kruskal-Wallis test implemented in SC3), or by
comparing gene expression between clusters in a pairwise manner. In the
following chapter we will mainly consider tools developed for pairwise
comparisons. 

These method may also be applied when comparing cells obtained from different
groups or conditions. Such analyses can be complicated by differing cell type
proportions between samples (i.e. distinct samples cell populations; the unit of
replication in the study). In such cases, it is likely beneficial to identify
distinct cell types and conduct differential expression testing between
conditions within each cell type.


### Differences in Distribution

Unlike bulk RNA-seq, we generally have a large number of samples (i.e. cells)
for each group we are comparing in single-cell experiments. Thus we may be able
to take advantage of the whole distribution of expression values in each group
to identify differences between groups rather than only comparing estimates of
mean-expression as is standard for bulk RNASeq.

There are two main approaches to comparing distributions. Firstly, we can use
existing statistical models/distributions and fit the same type of model to the
expression in each group then test for differences in the parameters for each
model, or test whether the model fits better if a particular parameter is allowed
to be different according to group. For instance in Chapter
\@ref(dealing-with-confounders) we used `edgeR` to test whether allowing mean
expression to be different in different batches significantly improved the fit
of a negative binomial model of the data.

Alternatively, we can use a non-parametric test which does not assume that
expression values follow any particular distribution, e.g. the
[Kolmogorov-Smirnov test
(KS-test)](https://en.wikipedia.org/wiki/Kolmogorov%E2%80%93Smirnov_test).
Non-parametric tests generally convert observed expression values to ranks and
test whether the distribution of ranks for one group are signficantly different
from the distribution of ranks for the other group. However, some non-parametric
methods fail in the presence of a large number of tied values, such as the case
for dropouts (zeros) in single-cell RNA-seq expression data. Moreover, if the
conditions for a parametric test hold, then it will typically be more powerful
than a non-parametric test.


### Benchmarking of DE methods for scRNA-seq data

So far there has been one high-quality benchmarking study of single-cell
differential expression methods [@Soneson2018-hy]. The figure below summarises
the results from that paper (which is well worth reading in full!):


```r
knitr::include_graphics("figures/soneson-de-benchmark-fig5.png")
```

<div class="figure" style="text-align: center">
<img src="figures/soneson-de-benchmark-fig5.png" alt="Figure 5 reproduced from Soneson and Robinson (2018). Summary of DE method performance across all major evaluation criteria. Criteria and cutoff values for performance categories are available in the Online Methods. Methods are ranked by their average performance across the criteria, with the numerical encoding good = 2, intermediate = 1, poor = 0. NODES and SAMseq do not return nominal P values and were therefore not evaluated in terms of the FPR." width="90%" />
<p class="caption">(\#fig:de-benchmarking)Figure 5 reproduced from Soneson and Robinson (2018). Summary of DE method performance across all major evaluation criteria. Criteria and cutoff values for performance categories are available in the Online Methods. Methods are ranked by their average performance across the criteria, with the numerical encoding good = 2, intermediate = 1, poor = 0. NODES and SAMseq do not return nominal P values and were therefore not evaluated in terms of the FPR.</p>
</div>

One particularly surprising outcome of this benchmarking study is that almost
all methods designed specifically for the analysis of scRNA-seq data are
outperformed by established bulk RNA-seq DE methods (edgeR, limma) and standard,
classical statistical methods (t-test, Wilcoxon rank-sum tests). MAST
[@Finak2015-ow] is the only method designed specifically for scRNA-seq data that
performs well in this benchmark. These benchmarking results are a credit to the
durability and flexibility of the leading bulk RNA-seq DE methods and a subtle
indictment of the land rush of new scRNA-seq methods that were published without
adequate comparison to existing bulk RNA-seq methods. 


### Models of single-cell RNA-seq data

The most common model of RNASeq data is the negative binomial model:



```r
set.seed(1)
hist(
    rnbinom(
        1000, 
        mu = 10, 
        size = 100), 
    col = "grey50", 
    xlab = "Read Counts", 
    main = "Negative Binomial"
)
```

<div class="figure" style="text-align: center">
<img src="de-intro_files/figure-html/nb-plot-1.png" alt="Negative Binomial distribution of read counts for a single gene across 1000 cells" width="90%" />
<p class="caption">(\#fig:nb-plot)Negative Binomial distribution of read counts for a single gene across 1000 cells</p>
</div>
Mean:
$\mu = mu$

Variance:
$\sigma^2 = mu + mu^2/size$

It is parameterized by the mean expression (mu) and the dispersion (size), which
is inversely related to the variance. The negative binomial model fits bulk
RNA-seq data very well and it is used for most statistical methods designed for
such data. In addition, it has been show to fit the distribution of molecule
counts obtained from data tagged by unique molecular identifiers (UMIs) quite
well ([Grun et al.
2014](http://www.nature.com/nmeth/journal/v11/n6/full/nmeth.2930.html), [Islam
et al. 2011](http://genome.cshlp.org/content/21/7/1160)).

However, a raw negative binomial model does not necessarily fit full-length
transcript data as well due to the high dropout rates relative to the non-zero
read counts. For this type of data a variety of zero-inflated negative binomial
models have been proposed (e.g.
[MAST](https://bioconductor.org/packages/release/bioc/html/MAST.html),
[SCDE](https://bioconductor.org/packages/release/bioc/html/scde.html)).


```r
d <- 0.5;
counts <- rnbinom(
    1000, 
    mu = 10, 
    size = 100
)
counts[runif(1000) < d] <- 0
hist(
    counts, 
    col = "grey50", 
    xlab = "Read Counts", 
    main = "Zero-inflated NB"
)
```

<div class="figure" style="text-align: center">
<img src="de-intro_files/figure-html/zero-inflation-plot-1.png" alt="Zero-inflated Negative Binomial distribution" width="90%" />
<p class="caption">(\#fig:zero-inflation-plot)Zero-inflated Negative Binomial distribution</p>
</div>
Mean:
$\mu = mu \cdot (1 - d)$

Variance:
$\sigma^2 = \mu \cdot (1-d) \cdot (1 + d \cdot \mu + \mu / size)$

These models introduce a new parameter $d$, for the dropout rate, to the
negative binomial model. As we saw in Chapter 19, the dropout rate of a gene is
strongly correlated with the mean expression of the gene. Different
zero-inflated negative binomial models use different relationships between mu
and d and some may fit $\mu$ and $d$ to the expression of each gene
independently.

Finally, several methods use a Poisson-Beta distribution which is based on a
mechanistic model of transcriptional bursting. There is strong experimental
support for this model ([Kim and Marioni,
2013](https://genomebiology.biomedcentral.com/articles/10.1186/gb-2013-14-1-r7))
and it provides a good fit to scRNA-seq data but it is less easy to use than the
negative-binomial models and much less existing methods upon which to build than
the negative binomial model.


```r
a <- 0.1
b <- 0.1
g <- 100
lambdas <- rbeta(1000, a, b)
counts <- sapply(g*lambdas, function(l) {rpois(1, lambda = l)})
hist(
    counts, 
    col = "grey50", 
    xlab = "Read Counts", 
    main = "Poisson-Beta"
)
```

<img src="de-intro_files/figure-html/pois-beta-plot-1.png" width="90%" style="display: block; margin: auto;" />
Mean:
$\mu = g \cdot a / (a + b)$

Variance:
$\sigma^2 = g^2 \cdot a \cdot b/((a + b + 1) \cdot (a + b)^2)$

This model uses three parameters: $a$ the rate of activation of transcription;
$b$ the rate of inhibition of transcription; and $g$ the rate of transcript
production while transcription is active at the locus. Differential expression
methods may test each of the parameters for differences across groups or only
one (often $g$).

All of these models may be further expanded to explicitly account for other
sources of gene expression differences such as batch-effect or library depth
depending on the particular DE algorithm.

__Exercise__: Vary the parameters of each distribution to explore how they
affect the distribution of gene expression. How similar are the Poisson-Beta and
Negative Binomial models?

