---
output: html_document
---



# Normalization, confounders and batch correction

## Normalization theory

### Introduction



In this chapter, we will explore approaches to _normalization_, _confounder
identification_ and _batch correction_ for scRNA-seq data.

Even in the absence of specific confounding factors, thoughtful _normalization_
of scRNA-seq data is required. The raw count values are not directly comparable
between cells, because in general the sequencing depth (number of reads
obtained; often called _library size_) is very different across
cells---orders-of-magnitude differences in sequencing depth are commonly
observed between cells in an scRNA-seq dataset. If ignored, or not handled
correctly, library size differences can be the dominant source of variation
between single-cell gene expression profiles, obscuring the biological signal of
interest. 
n
Related to library size, differences in _library composition_ can also cause
problems when we are trying to compare expression profiles between cells.
Normalization can and should also account for differences in library
composition.

In addition to normalization, it is also useful to _identify confounding
factors_ so that they can be accounted for in downstream analyses. In many
cases, "accounting" for confounding variables may involve incorporating them as
variables in a particular statistical model (e.g. in a differential expression
model). In other cases, it may be desirable to "regress out" (either in a
literal or figurative sense) confounding factors---the challenge for scRNA-seq
data is finding the right model and/or data transformation such that regressing
out confounding factors would work as desired. We discuss this further below.

The issue of _batch effects_ is just as important for scRNA-seq data as it is in
other areas of genomics. Briefly, scRNA-seq and other 'omics assays are
sensitive to minor differences in technical features of data generation. As
such, even when assaying the same experimental or biological system,
measurements taken at difference times and places or by different people will
differ substantially. To make valid comparisons between cells, samples or
groups, we first need to design our studies to be robust to batch effects and
then we need to treat batch effects appropriately in our analyses.

In the following sections, we will explore simple size-factor normalizations
correcting for library size and composition and also discuss a more recent,
conceptually quite different, approach to tackling the problem of library size
differences between cells.

### Library size

Library sizes vary because scRNA-seq data is often sequenced on highly
multiplexed platforms the total reads which are derived from each cell may
differ substantially. 

Most scRNA-seq platforms and/or quantification methods currently available
produce count values as the "raw", "observed", gene expression values. For such
count data, the library size must be corrected for as part of data normalization. 

One popular strategy, borrowed and extended from the analysis of bulk RNA-seq
data, is to multiply or divide each column of the expression matrix (in our
setup columns correspond to cells) by a __"normalization factor"__ which is an
estimate of the library size relative to the other cells. Many methods to
correct for library size have been developed for bulk RNA-seq and can be equally
applied to scRNA-seq (eg. __UQ__, __SF__, __CPM__, __RPKM__, __FPKM__, __TPM__).
In addition, single-cell specific size-factor normalization methods have been
proposed to better handle the characteristics of scRNA-seq data (namely greater
sparsity/proportion of zero counts). We will demonstrate use of the size-factor
normalization method from the `scran` package in this chapter.

A conceptually different approach to normalization of scRNA-seq data was
proposed earlier in 2019 by [@Hafemeister2019-rx]. The idea behind the
`sctransform` approach is to fit a regularized negative binomial model to the
raw count data, with library size as the only explanatory variable in the model.
The residuals from this model can then be used as normalized and
variance-stabilized expression values. We show the use of this method too in
this chapter.

Some quantification methods (particularly those that quantify transcript-level
expression, e.g. [`Salmon`](https://combine-lab.github.io/salmon/),
[`kallisto`](https://pachterlab.github.io/kallisto/)) return
transcripts-per-million values, TPM (instead of or in addition to count values),
which effectively incorporate library size when determining gene expression
estimates and thus do not require subsequent normalization for library size.
However, TPM values may still be susceptible to _library composition_ biases and
so normalization may still be required.


### Scaling or size-factor normalization methods

The normalization methods discussed in this section all involve dividing the
counts for each cell by a constant value to account for library size and, in
some cases, library composition. These methods will typically give
(adjusted/normalized) counts-per-million (CPM) or transcripts-per-million (TPM)
values. 

Ideally, after applying one of these scaling/size-factor normalization methods,
the CPM/TPM values produced are comparable across cells, with the effects of
sequencing depth removed. However, even if this is true (i.e. the normalization
has worked well), the CPM/TPM values do not have _stable variance_.
Specifically, as the size of the values increases, so does the variance. This
feature of the data (heteroskedacity, or asymmetric, heavy-tailed distributions)
is problematic for statistical analysis methods that assume homoskedacity, that
is that there is no relationship between the mean of expression values and their
variance (i.e. just about anything that uses a Gaussian error model). 

As such, we should apply a _variance stabilizing transformation_ to these data
so that we can use standard statistical methods like linear regression and PCA
with confidence. Developing a thoroughly effective variance stabilizing
transformation is a challenge, so almost universally a log transformation
(typically log2) is applied to the CPM/TPM values (the `logcounts` slot in a
`SingleCellExperiment` object is expected to contain (normalized) log2-scale
CPM/TPM values). For high-depth cells and highly-expressed genes this
transformation generally works well (as for bulk RNA-seq data), but, as we will
discuss below, it often performs sub-optimally for (sparse) scRNA-seq data.

#### CPM

The simplest way to normalize this data is to convert it to counts per million
(__CPM__) by dividing each column by its total then multiplying by 1,000,000.
Note that spike-ins should be excluded from the calculation of total expression
in order to correct for total cell RNA content, therefore we will only use
endogenous genes. Example of a __CPM__ function in `R` (using the `scater`
package):


```r
calc_cpm <-
function (expr_mat, spikes = NULL) 
{
    norm_factor <- colSums(expr_mat[-spikes, ])
    return(t(t(expr_mat)/norm_factor)) * 10^6
}
```

One potential drawback of __CPM__ is if your sample contains genes that are both
very highly expressed and differentially expressed across the cells. In this
case, the total molecules in the cell may depend of whether such genes are
on/off in the cell and normalizing by total molecules may hide the differential
expression of those genes and/or falsely create differential expression for the
remaining genes.

__Note__ __RPKM__, __FPKM__ and __TPM__ are variants on __CPM__ which further
adjust counts by the length of the respective gene/transcript. TPM is usually a
direct output of a transcript expression quantification method (e.g. Salmon,
kallisto, etc).

To deal with this potentiality several other measures were devised.

#### RLE (SF)

The __size factor (SF)__ was proposed and popularized by DESeq [@Anders2010-jr].
First the geometric mean of each gene across all cells is calculated. The size
factor for each cell is the median across genes of the ratio of the expression
to the gene's geometric mean. A drawback to this method is that since it uses
the geometric mean only genes with non-zero expression values across all cells
can be used in its calculation, making it unadvisable for large low-depth
scRNASeq experiments. `edgeR` & `scater` call this method __RLE__ for "relative
log expression" (to distinguish it from the many other size-factor normalization
methods that now exist). Example of a __SF__ function in `R` (from the `edgeR`
package):


```r
calc_sf <-
function (expr_mat, spikes = NULL) 
{
    geomeans <- exp(rowMeans(log(expr_mat[-spikes, ])))
    SF <- function(cnts) {
        median((cnts/geomeans)[(is.finite(geomeans) & geomeans > 
            0)])
    }
    norm_factor <- apply(expr_mat[-spikes, ], 2, SF)
    return(t(t(expr_mat)/norm_factor))
}
```

#### UQ

The __upperquartile (UQ)__ was proposed by [@Bullard2010-eb]. Here each column
is divided by the 75% quantile of the counts for each library. Often the
calculated quantile is scaled by the median across cells to keep the absolute
level of expression relatively consistent. A drawback to this method is that for
low-depth scRNASeq experiments the large number of undetected genes may result
in the 75% quantile being zero (or close to it). This limitation can be overcome
by generalizing the idea and using a higher quantile (eg. the 99% quantile is
the default in scater) or by excluding zeros prior to calculating the 75%
quantile. Example of a __UQ__ function in `R` (again from the `edgeR` package):


```r
calc_uq <-
function (expr_mat, spikes = NULL) 
{
    UQ <- function(x) {
        quantile(x[x > 0], 0.75)
    }
    uq <- unlist(apply(expr_mat[-spikes, ], 2, UQ))
    norm_factor <- uq/median(uq)
    return(t(t(expr_mat)/norm_factor))
}
```

#### TMM

Another method is called __TMM__ is the weighted trimmed mean of M-values (to
the reference) proposed by [@Robinson2010-hz]. The M-values in question are the
gene-wise log2-fold changes between individual cells. One cell is used as the
reference then the M-values for each other cell is calculated compared to this
reference. These values are then trimmed by removing the top and bottom ~30%,
and the average of the remaining values is calculated by weighting them to
account for the effect of the log scale on variance. Each non-reference cell is
multiplied by the calculated factor. Two potential issues with this method are
insufficient non-zero genes left after trimming, and the assumption that most
genes are not differentially expressed.


```r
sizeFactors(umi.qc) <- edgeR::calcNormFactors(counts(umi.qc), method = "TMM")
```


#### scran

`scran` package implements a variant on __CPM__ size-factor normalization
specialized for single-cell data [@L_Lun2016-pq]. Briefly this method deals with
the problem of vary large numbers of zero values per cell by pooling cells
together calculating a normalization factor (similar to __TMM__) for the sum of
each pool. Since each cell is found in many different pools, cell-specific
factors can be deconvoluted from the collection of pool-specific factors using
linear algebra.

This method applies a "quick cluster" method to get rough clusters of cells to
pool together to apply the strategy outlined above.


```r
qclust <- quickCluster(umi.qc, min.size = 30)
umi.qc <- computeSumFactors(umi.qc, sizes = 15, clusters = qclust)
```

### `sctransform`

The `sctransform` method is very different from the scaling/size-factor methods
discussed above. In their paper, [@Hafemeister2019-rx] argue that the
log-transformation of (normalized) CPM values does not stabilise the variance of
expression values, particularly in the case of sparse(r) UMI-count data. Figure
1 of their paper (reproduced below) sets out this argument that strong
relationships exist between gene expression and total cell UMI count, even after
applying a scaled log-normalization method.

<div class="figure" style="text-align: center">
<img src="./figures/sctransform_fig1.png" alt="Reproduction of Figure 1 from Hafemeister and Satija (2019). 33,148 PBMC dataset from 10x genomics. A) Distribution of total UMI counts / cell (’sequencing depth’). B) We placed genesinto six groups, based on their average expression in the dataset. C) For each gene group, we examined the average relationship between observed counts and cell sequencing depth. We fit a smooth line for each gene individually and combined results based on the groupings in (B). Black line shows mean, colored region indicates interquartile range. D) Same as in (C), but showing scaled log-normalized values instead of UMI counts. Values were scaled (z-scored) so that a single y-axis range could be used. E) Relationship between gene variance and cell sequencing depth; Cells were placed into five equal-sized groups based on total UMI counts (group 1 has the greatest depth), and we calculated the total variance of each gene group within each bin. For effectively normalized data, each cell bin should contribute 20% to the variance of each gene group." width="90%" />
<p class="caption">(\#fig:unnamed-chunk-3)Reproduction of Figure 1 from Hafemeister and Satija (2019). 33,148 PBMC dataset from 10x genomics. A) Distribution of total UMI counts / cell (’sequencing depth’). B) We placed genesinto six groups, based on their average expression in the dataset. C) For each gene group, we examined the average relationship between observed counts and cell sequencing depth. We fit a smooth line for each gene individually and combined results based on the groupings in (B). Black line shows mean, colored region indicates interquartile range. D) Same as in (C), but showing scaled log-normalized values instead of UMI counts. Values were scaled (z-scored) so that a single y-axis range could be used. E) Relationship between gene variance and cell sequencing depth; Cells were placed into five equal-sized groups based on total UMI counts (group 1 has the greatest depth), and we calculated the total variance of each gene group within each bin. For effectively normalized data, each cell bin should contribute 20% to the variance of each gene group.</p>
</div>

One effect of the failure of the scaled log-normalization to remove the
relationship between total cell UMI count and expression is that
dimension-reduction methods (especially PCA) applied to the log-normalized data
can return reduced dimension spaces where, very often, the first dimension is
highly correlated with total cell UMI count to total cell genes expressed. This
effect is noted and discussed by [@William_Townes2019-wg].

The `sctransform` solution is to fit a negative binomial (NB) generalized linear
model to the UMI counts for each gene, with an intercept term and a coefficient
for library size (specifically using log10(total cell UMI count) as a covariate)
as parameters in the model. The negative binomial model can account for much
more variance in the observed count data than a simpler model like the Poisson
can. To avoid overfitting the model to the data, the gene-wise intercept,
library size and overdispersion parameters are regularized by fitting a loess
(locally-linear smoothing method) to the per-gene estimates from the GLM.

<div class="figure" style="text-align: center">
<img src="./figures/sctransform_fig2A.png" alt="Reproduction of Figure 2A from Hafemeister and Satija (2019). They fit NB regression models for each gene individually, and bootstrapped the process to measure uncertainty in the resulting parameter estimates. A) Model parameters for 16,809 genes for the NB regression model, plotted as a function of average gene abundance. The color of each point indicates a parameter uncertainty score as determined by bootstrapping (Methods). Pink line shows the regularized parameters obtained via kernel regression." width="90%" />
<p class="caption">(\#fig:unnamed-chunk-4)Reproduction of Figure 2A from Hafemeister and Satija (2019). They fit NB regression models for each gene individually, and bootstrapped the process to measure uncertainty in the resulting parameter estimates. A) Model parameters for 16,809 genes for the NB regression model, plotted as a function of average gene abundance. The color of each point indicates a parameter uncertainty score as determined by bootstrapping (Methods). Pink line shows the regularized parameters obtained via kernel regression.</p>
</div>

The regularized NB GLM is presented as an attractive middle ground between the
(underfit) Poisson model and the (overfit) unregularized NB model. The Pearson
residuals from the regularized NB GLM are used as "normalized" expression values
for downstream analyses.

<div class="figure" style="text-align: center">
<img src="./figures/sctransform_fig5.png" alt="Reproduction of Figure 4 from Hafemeister and Satija (2019). A) For four genes, we show the relationship between cell sequencing depth and molecular counts. White points show the observed data. Background color represents the Pearson residual magnitude under three error models. For MALAT1 (does not vary across cell types) the Poisson error model does not account for overdispersion, and incorrectly infers significant residual variation (biological heterogeneity). For S100A9 (a CD14+ Monocyte marker) and CD74 (expressed in antigen-presenting cells) the non-regularized NB model overfits the data, and collapses biological heterogeneity. For PPBP (a Megakaryocyte marker) both non-regularized models wrongly fit a negative slope. B) Boxplot of Pearson residuals for models shown in A. X-axis range shown is limited to [-8, 25] for visual clarity." width="90%" />
<p class="caption">(\#fig:unnamed-chunk-5)Reproduction of Figure 4 from Hafemeister and Satija (2019). A) For four genes, we show the relationship between cell sequencing depth and molecular counts. White points show the observed data. Background color represents the Pearson residual magnitude under three error models. For MALAT1 (does not vary across cell types) the Poisson error model does not account for overdispersion, and incorrectly infers significant residual variation (biological heterogeneity). For S100A9 (a CD14+ Monocyte marker) and CD74 (expressed in antigen-presenting cells) the non-regularized NB model overfits the data, and collapses biological heterogeneity. For PPBP (a Megakaryocyte marker) both non-regularized models wrongly fit a negative slope. B) Boxplot of Pearson residuals for models shown in A. X-axis range shown is limited to [-8, 25] for visual clarity.</p>
</div>

The regularized NB GLM also provides a natural way to do feature selection (i.e.
find informative genes) using the deviance of the fitted GLM for each gene. We
discuss this further in the [Feature Selection](#feature-selection) section.

We find the Pearson residuals from `sctransform` to be highly suitable as input
to visualisation (dimension reduction) and clustering methods. For several other
analyses (e.g. differential expression analyses), where statistical models
designed for sparse count data are available, we prefer to use approaches that
work with the "raw" count data. We are not yet sure how well `sctransform`
performs on full-length transcript (i.e. non-UMI) count data.


### Downsampling

A final way to correct for library size is to downsample the expression matrix
so that each cell has approximately the same total number of molecules. The
benefit of this method is that zero values will be introduced by the down
sampling thus eliminating any biases due to differing numbers of detected genes.
However, the major drawback is that the process is not deterministic so each
time the downsampling is run the resulting expression matrix is slightly
different. Thus, often analyses must be run on multiple downsamplings to ensure
results are robust. 

Downsampling to the depth of the cell with the lowest sequencing depth (that
still passes QC) will typically discard much (most) of the information gathered
in a (typically expensive) scRNA-seq experiment. We view this as a heavy price
to pay for a normalization method that generally does not seem to outperform
alternatives. Thus, we would not recommend downsampling as a normalization
strategy for scRNA-seq data unless all alternatives have failed.


### Effectiveness

To compare the efficiency of different normalization methods we will use visual
inspection of `PCA` plots and calculation of cell-wise _relative log expression_
via `scater`'s `plotRLE()` function. Namely, cells with many (few) reads have
higher (lower) than median expression for most genes resulting in a positive
(negative) _RLE_ across the cell, whereas normalized cells have an _RLE_ close
to zero. Example of a _RLE_ function in `R`:


```r
calc_cell_RLE <-
function (expr_mat, spikes = NULL) 
{
    RLE_gene <- function(x) {
        if (median(unlist(x)) > 0) {
            log((x + 1)/(median(unlist(x)) + 1))/log(2)
        }
        else {
            rep(NA, times = length(x))
        }
    }
    if (!is.null(spikes)) {
        RLE_matrix <- t(apply(expr_mat[-spikes, ], 1, RLE_gene))
    }
    else {
        RLE_matrix <- t(apply(expr_mat, 1, RLE_gene))
    }
    cell_RLE <- apply(RLE_matrix, 2, median, na.rm = T)
    return(cell_RLE)
}
```

__Note__ The __RLE__, __TMM__, and __UQ__ size-factor methods were developed for
bulk RNA-seq data and, depending on the experimental context, may not be
appropriate for single-cell RNA-seq data, as their underlying assumptions may be
problematically violated.

__Note__ The `calcNormFactors` function from the `edgeR` package implements
several library size normalization methods making it easy to apply any of these
methods to our data.

__Note__ `edgeR` makes extra adjustments to some of the normalization methods
which may result in somewhat different results than if the original methods are
followed exactly, e.g. edgeR's and scater's "RLE" method which is based on the
"size factor" used by [DESeq](http://bioconductor.org/packages/DESeq) may give
different results to the `estimateSizeFactorsForMatrix` method in the
`DESeq`/`DESeq2` packages. In addition, some (earlier) versions of `edgeR` will
not calculate the normalization factors correctly unless `lib.size` is set at 1
for all cells.

__Note__ For __CPM__ normalisation we use `scater`'s `calculateCPM()` function.
For __RLE__, __UQ__ and __TMM__ we used to use `scater`'s `normaliseExprs()`
function, but it was deprecated and has been removed from the package). For
__scran__ we use the `scran` package to calculate size factors (it also operates
on `SingleCellExperiment` class) and `scater`'s `normalize()` to normalise the
data. All these normalization functions save the results to the `logcounts` slot
of the `SCE` object. 

## Normalization practice (UMI)

We will continue to work with the `tung` data that was used in the previous
chapter.


```r
library(scRNA.seq.funcs)
library(scater)
library(scran)
options(stringsAsFactors = FALSE)
set.seed(1234567)
umi <- readRDS("data/tung/umi.rds")
umi.qc <- umi[rowData(umi)$use, colData(umi)$use]
endog_genes <- !rowData(umi.qc)$is_feature_control
```

### Raw


```r
tmp <- runPCA(
  umi.qc[endog_genes, ],
  exprs_values = "logcounts_raw"
)
plotPCA(
    tmp,
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
)  + ggtitle("PCA plot: raw log-counts") 
```

<div class="figure" style="text-align: center">
<img src="exprs-norm_files/figure-html/norm-pca-raw-1.png" alt="PCA plot of the tung data" width="90%" />
<p class="caption">(\#fig:norm-pca-raw)PCA plot of the tung data</p>
</div>

### CPM


```r
logcounts(umi.qc) <- log2(calculateCPM(umi.qc, use_size_factors = FALSE) + 1)
plotPCA(
    umi.qc[endog_genes, ],
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
) + ggtitle("PCA plot: log2(CPM) values") 
```

<div class="figure" style="text-align: center">
<img src="exprs-norm_files/figure-html/norm-pca-cpm-1.png" alt="PCA plot of the tung data after CPM normalisation" width="90%" />
<p class="caption">(\#fig:norm-pca-cpm)PCA plot of the tung data after CPM normalisation</p>
</div>


```r
plotRLE(
    umi.qc[endog_genes, ], 
    exprs_values = "logcounts_raw",
    colour_by = "batch"
) + ggtitle("RLE plot: raw log-counts")
```

<div class="figure" style="text-align: center">
<img src="exprs-norm_files/figure-html/norm-ours-rle-cpm-1.png" alt="Cell-wise RLE of the tung data. The relative log expression profile of each cell is represented by a boxplot, which appears as a line here. The grey bar in the middle for each cell represent the interquartile range of the RLE values; the coloured lines represent the whiskers ofof a boxplot and extend above and below the grey bar by 1.5 times the interquartile range. The median RLE value is shown with a circle." width="90%" />
<p class="caption">(\#fig:norm-ours-rle-cpm1)Cell-wise RLE of the tung data. The relative log expression profile of each cell is represented by a boxplot, which appears as a line here. The grey bar in the middle for each cell represent the interquartile range of the RLE values; the coloured lines represent the whiskers ofof a boxplot and extend above and below the grey bar by 1.5 times the interquartile range. The median RLE value is shown with a circle.</p>
</div>

```r
plotRLE(
    umi.qc[endog_genes, ], 
    exprs_values = "logcounts",
    colour_by = "batch"
) + ggtitle("RLE plot: log2(CPM)")
```

<div class="figure" style="text-align: center">
<img src="exprs-norm_files/figure-html/norm-ours-rle-cpm-2.png" alt="Cell-wise RLE of the tung data. The relative log expression profile of each cell is represented by a boxplot, which appears as a line here. The grey bar in the middle for each cell represent the interquartile range of the RLE values; the coloured lines represent the whiskers ofof a boxplot and extend above and below the grey bar by 1.5 times the interquartile range. The median RLE value is shown with a circle." width="90%" />
<p class="caption">(\#fig:norm-ours-rle-cpm2)Cell-wise RLE of the tung data. The relative log expression profile of each cell is represented by a boxplot, which appears as a line here. The grey bar in the middle for each cell represent the interquartile range of the RLE values; the coloured lines represent the whiskers ofof a boxplot and extend above and below the grey bar by 1.5 times the interquartile range. The median RLE value is shown with a circle.</p>
</div>

**Q:** How well would you say the two approaches above normalize the data?

### scran

`scran`'s method for size-factor estimation will almost always be preferable for
scRNA-seq data to methods that were developed for bulk RNA-seq data (TMM, RLE,
UQ). Thus, we will just demonstrate the use of `scran` size-factor normalization
here as representative of size-factor normalization more generally.

The code below computes the size factors and then the `normalize()` function in
`scater` applies those size factors along with the library sizes to the count
matrix to produce normalized log2-counts-per-million values that are then stored
in the `logcounts` slot of the `SingleCellExperiment` object.


```r
qclust <- quickCluster(umi.qc, min.size = 30, use.ranks = FALSE)
umi.qc <- computeSumFactors(umi.qc, sizes = 15, clusters = qclust)
umi.qc <- normalize(umi.qc)
plotPCA(
    umi.qc[endog_genes, ],
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
) + ggtitle("PCA plot: scran size-factor normalization") 
```

<div class="figure" style="text-align: center">
<img src="exprs-norm_files/figure-html/norm-pca-lsf-1.png" alt="PCA plot of the tung data after LSF normalisation" width="90%" />
<p class="caption">(\#fig:norm-pca-lsf)PCA plot of the tung data after LSF normalisation</p>
</div>


```r
plotRLE(
    umi.qc[endog_genes, ], 
    exprs_values = "logcounts",
    colour_by = "batch"
) + ggtitle("RLE plot: scran size-factor normalization")
```

<div class="figure" style="text-align: center">
<img src="exprs-norm_files/figure-html/norm-ours-rle-scran-1.png" alt="Cell-wise RLE of the tung data" width="90%" />
<p class="caption">(\#fig:norm-ours-rle-scran)Cell-wise RLE of the tung data</p>
</div>

`scran` sometimes calculates negative or zero size factors. These will
completely distort the normalized expression matrix. We can check the size
factors scran has computed like so:



```r
summary(sizeFactors(umi.qc))
```

```
##    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
##  0.4836  0.7747  0.9532  1.0000  1.1483  3.2873
```

For this dataset all the size factors are reasonable so we are done. If you find
scran has calculated negative size factors try increasing the cluster and pool
sizes until they are all positive.

We sometimes filter out cells with very large size-factors (you may like to
think about why), but we will not demonstrate that here.


### sctransform

The `sctransform` approach to using Pearson residuals from an regularized
negative binomial generalized linear model was introduced above. Here we
demonstrate how to apply this method.

Note that (due to what looks like a bug in this version of `sctransform`) we
need to convert the UMI count matrix to a sparse format to apply sctransform.



```r
umi_sparse <- as(counts(umi.qc), "dgCMatrix")
### Genes expressed in at least 5 cells will be kept
sctnorm_data <- sctransform::vst(umi = umi_sparse, min_cells = 1,
                                 cell_attr = as.data.frame(colData(umi.qc)),
                                 latent_var = "log10_total_counts_endogenous")
```


```r
## Pearson residuals, or deviance residuals
dim(sctnorm_data$y)
```

```
## [1] 14066   657
```

```r
dim(umi.qc)
```

```
## [1] 14066   657
```

```r
sctnorm_data$model_str
```

```
## [1] "y ~ log10_total_counts_endogenous"
```

```r
assay(umi.qc, "sctrans_norm") <- sctnorm_data$y
```

Let us look at the NB GLM model parameters estimated by sctransform.


```r
#sce$log10_total_counts
##       Matrix of estimated model parameters per gene (theta and regression coefficients)
sctransform::plot_model_pars(sctnorm_data)
```

<img src="exprs-norm_files/figure-html/sctransform-params-plot-1.png" width="90%" style="display: block; margin: auto;" />

We can look at the effect of sctransform's normalization on three particular
genes, ACTB, POU5F1 (aka OCT4) and CD74.


```r
##c('ACTB', 'Rpl10', 'Cd74')
genes_plot <- c("ENSG00000075624", "ENSG00000204531", "ENSG00000019582")
sctransform::plot_model(sctnorm_data, umi_sparse, genes_plot, 
                        plot_residual = TRUE, cell_attr = as.data.frame(colData(umi.qc)))
```

<img src="exprs-norm_files/figure-html/sctransform-genes-plot-1.png" width="90%" style="display: block; margin: auto;" />


```r
reducedDim(umi.qc, "PCA_sctrans_norm") <- reducedDim(
  runPCA(umi.qc[endog_genes, ], exprs_values = "sctrans_norm")
)
plotReducedDim(
  umi.qc,
  use_dimred = "PCA_sctrans_norm",
  colour_by = "batch",
  size_by = "total_features_by_counts",
  shape_by = "individual"
) + ggtitle("PCA plot: sctransform normalization") 
```

<div class="figure" style="text-align: center">
<img src="exprs-norm_files/figure-html/norm-pca-sctransform-1.png" alt="PCA plot of the tung data after sctransform normalisation (Pearson residuals)." width="90%" />
<p class="caption">(\#fig:norm-pca-sctransform)PCA plot of the tung data after sctransform normalisation (Pearson residuals).</p>
</div>


```r
plotRLE(
    umi.qc[endog_genes, ], 
    exprs_values = "sctrans_norm",
    colour_by = "batch"
) + ggtitle("RLE plot: sctransform normalization")
```

<div class="figure" style="text-align: center">
<img src="exprs-norm_files/figure-html/norm-ours-rle-sctransform-1.png" alt="Cell-wise RLE of the tung data" width="90%" />
<p class="caption">(\#fig:norm-ours-rle-sctransform)Cell-wise RLE of the tung data</p>
</div>



### Normalisation for gene/transcript length

Some methods combine library size and fragment/gene length normalization such
as:

* __RPKM__ - Reads Per Kilobase Million (for single-end sequencing)
* __FPKM__ - Fragments Per Kilobase Million (same as __RPKM__ but for paired-end
  sequencing, makes sure that paired ends mapped to the same fragment are not
  counted twice)
* __TPM__ - Transcripts Per Kilobase Million (same as __RPKM__, but the order of
  normalizations is reversed - length first and sequencing depth second)

These methods are not applicable to our dataset since the end
of the transcript which contains the UMI was preferentially
sequenced. Furthermore in general these should only be calculated
using appropriate quantification software from aligned BAM files not
from read counts since often only a portion of the entire
gene/transcript is sequenced, not the entire length. If in doubt check 
for a relationship between gene/transcript length and expression level.

However, here we show how these normalisations can be calculated using `scater`.
First, we need to find the effective transcript length in Kilobases. However,
our dataset containes only gene IDs, therefore we will be using the gene lengths
instead of transcripts. `scater` uses the
[biomaRt](https://bioconductor.org/packages/release/bioc/html/biomaRt.html)
package, which allows one to annotate genes by other attributes:


```r
umi.qc <- getBMFeatureAnnos(
    umi.qc,
    filters = "ensembl_gene_id", 
    attributes = c(
        "ensembl_gene_id",
        "hgnc_symbol",
        "chromosome_name",
        "start_position",
        "end_position"
    ), 
    biomart = "ENSEMBL_MART_ENSEMBL", 
    dataset = "hsapiens_gene_ensembl",
    host = "www.ensembl.org"
)

# If you have mouse data, change the arguments based on this example:
# getBMFeatureAnnos(
#     object,
#     filters = "ensembl_transcript_id",
#     attributes = c(
#         "ensembl_transcript_id",
#         "ensembl_gene_id", 
#         "mgi_symbol",
#         "chromosome_name",
#         "transcript_biotype",
#         "transcript_start",
#         "transcript_end",
#         "transcript_count"
#     ),
#     biomart = "ENSEMBL_MART_ENSEMBL",
#     dataset = "mmusculus_gene_ensembl",
#     host = "www.ensembl.org"
# )
```

Some of the genes were not annotated, therefore we filter them out:


```r
umi.qc.ann <- umi.qc[!is.na(rowData(umi.qc)$ensembl_gene_id), ]
```

Now we compute the total gene length in Kilobases by using the `end_position`
and `start_position` fields:


```r
eff_length <- 
    abs(rowData(umi.qc.ann)$end_position - rowData(umi.qc.ann)$start_position) / 1000
```


```r
plot(eff_length, rowMeans(counts(umi.qc.ann)))
```

There is no relationship between gene length and mean expression so __FPKM__s &
__TPM__s are inappropriate for this dataset. This is what we would expect for
UMI protocols that tag one end of the transcript. But we will demonstrate them
anyway.

__Note__ Here calculate the total gene length instead of the total exon length.
Many genes will contain lots of introns so their `eff_length` will be very
different from what we have calculated. Please consider our calculation as
approximation. If you want to use the total exon lengths, please refer to [this
page](https://www.biostars.org/p/83901/).

Now we are ready to perform the normalisations:


```r
tpm(umi.qc.ann) <- log2(calculateTPM(umi.qc.ann, eff_length) + 1)
```

Plot the results as a PCA plot:

```r
tmp <- runPCA(
  umi.qc.ann,
  exprs_values = "tpm",
)
plotPCA(
    tmp,
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
)
```


```r
tpm(umi.qc.ann) <- log2(calculateFPKM(umi.qc.ann, eff_length) + 1)
```


```r
tmp <- runPCA(
  umi.qc.ann,
  exprs_values = "tpm",
)
plotPCA(
    tmp,
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
)
```

__Note__ The `PCA` looks for differences between cells. Gene length is the same
across cells for each gene thus __FPKM__ is almost identical to the __CPM__ plot
(it is just rotated) since it performs __CPM__ first then normalizes gene
length. Whereas, __TPM__ is different because it weights genes by their length
before performing __CPM_**.

### Reflection

**Q:** What is your assessment of the performance of these different
normalization methods on the data presented here?

**Q:** Which normalization method would you prefer for this dataset? Why?


### Exercise

Perform the same analysis with read counts of the `tung` data. Use
`tung/reads.rds` file to load the reads `SCE` object. Once you have finished
please compare your results to ours (next chapter).

### sessionInfo()


```
## R version 3.6.0 (2019-04-26)
## Platform: x86_64-pc-linux-gnu (64-bit)
## Running under: Ubuntu 18.04.3 LTS
## 
## Matrix products: default
## BLAS:   /usr/lib/x86_64-linux-gnu/blas/libblas.so.3.7.1
## LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.7.1
## 
## locale:
##  [1] LC_CTYPE=en_AU.UTF-8       LC_NUMERIC=C              
##  [3] LC_TIME=en_AU.UTF-8        LC_COLLATE=en_AU.UTF-8    
##  [5] LC_MONETARY=en_AU.UTF-8    LC_MESSAGES=en_AU.UTF-8   
##  [7] LC_PAPER=en_AU.UTF-8       LC_NAME=C                 
##  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
## [11] LC_MEASUREMENT=en_AU.UTF-8 LC_IDENTIFICATION=C       
## 
## attached base packages:
## [1] parallel  stats4    stats     graphics  grDevices utils     datasets 
## [8] methods   base     
## 
## other attached packages:
##  [1] scran_1.12.1                scater_1.12.2              
##  [3] ggplot2_3.2.1               SingleCellExperiment_1.6.0 
##  [5] SummarizedExperiment_1.14.1 DelayedArray_0.10.0        
##  [7] BiocParallel_1.18.1         matrixStats_0.55.0         
##  [9] Biobase_2.44.0              GenomicRanges_1.36.1       
## [11] GenomeInfoDb_1.20.0         IRanges_2.18.3             
## [13] S4Vectors_0.22.1            BiocGenerics_0.30.0        
## [15] scRNA.seq.funcs_0.1.0      
## 
## loaded via a namespace (and not attached):
##  [1] viridis_0.5.1            dynamicTreeCut_1.63-1   
##  [3] edgeR_3.26.8             BiocSingular_1.0.0      
##  [5] viridisLite_0.3.0        DelayedMatrixStats_1.6.1
##  [7] elliptic_1.4-0           moments_0.14            
##  [9] assertthat_0.2.1         statmod_1.4.32          
## [11] highr_0.8                dqrng_0.2.1             
## [13] GenomeInfoDbData_1.2.1   vipor_0.4.5             
## [15] yaml_2.2.0               globals_0.12.4          
## [17] pillar_1.4.2             lattice_0.20-38         
## [19] glue_1.3.1               limma_3.40.6            
## [21] digest_0.6.21            XVector_0.24.0          
## [23] colorspace_1.4-1         plyr_1.8.4              
## [25] cowplot_1.0.0            htmltools_0.3.6         
## [27] Matrix_1.2-17            pkgconfig_2.0.3         
## [29] listenv_0.7.0            bookdown_0.13           
## [31] zlibbioc_1.30.0          purrr_0.3.2             
## [33] scales_1.0.0             Rtsne_0.15              
## [35] tibble_2.1.3             withr_2.1.2             
## [37] lazyeval_0.2.2           magrittr_1.5            
## [39] crayon_1.3.4             evaluate_0.14           
## [41] future_1.14.0            MASS_7.3-51.1           
## [43] beeswarm_0.2.3           tools_3.6.0             
## [45] stringr_1.4.0            locfit_1.5-9.1          
## [47] munsell_0.5.0            irlba_2.3.3             
## [49] orthopolynom_1.0-5       compiler_3.6.0          
## [51] rsvd_1.0.2               contfrac_1.1-12         
## [53] rlang_0.4.0              grid_3.6.0              
## [55] RCurl_1.95-4.12          BiocNeighbors_1.2.0     
## [57] igraph_1.2.4.1           labeling_0.3            
## [59] bitops_1.0-6             rmarkdown_1.15          
## [61] codetools_0.2-16         hypergeo_1.2-13         
## [63] gtable_0.3.0             deSolve_1.24            
## [65] reshape2_1.4.3           R6_2.4.0                
## [67] gridExtra_2.3            knitr_1.25              
## [69] dplyr_0.8.3              future.apply_1.3.0      
## [71] stringi_1.4.3            ggbeeswarm_0.6.0        
## [73] Rcpp_1.0.2               sctransform_0.2.0       
## [75] tidyselect_0.2.5         xfun_0.9
```

