---
output: html_document
---



# Latent spaces 

In many cases we may like to think of cells sitting in a low-dimensional,
"latent" space that captures relationships between cells more intuitively than
the very high-dimensional gene expression space.


```r
library(scater)
library(SingleCellExperiment)
library(glmpca)
library(ggplot2)
library(Polychrome)
library(slalom)
```


## Dimensionality reduction

__Why?__ \
- Reduce Curse of Dimensionality problems\
- Increase storage and computational efficiency\
- Visualize Data in 2D or 3D\


__Difficulty:__ Need to decide how many dimension to keep. 

### PCA: Principal component analysis

#### (traditional) PCA

PCA is a linear feature extraction technique. It performs a linear mapping of
the data to a lower-dimensional space in such a way that the variance of the
data in the low-dimensional representation is maximized. It does so by
calculating the eigenvectors from the covariance matrix. The eigenvectors that
correspond to the largest eigenvalues (the principal components) are used to
reconstruct a significant fraction of the variance of the original data.

In simpler terms, PCA combines your input features in a specific way that you
can drop the least important feature while still retaining the most valuable
parts of all of the features. As an added benefit, each of the new features or
components created after PCA are all independent of one another.

##### Basic ideas of PCA

- __Idea1: Dropping dimensions = Projection onto lower dimensional space__

<center> ![](figures/projection.png){width=30%} </center>

 &nbsp;  &nbsp;   Which dimension should we keep? \

- __Idea2: more variantion = more information__

 &nbsp;  &nbsp;   But what if the plot is not readily to be projected onto either X or Y-axis? 
 <center> ![](figures/rotation1.png){width=30%} </center>

##### Steps of PCA
- __Step1: Rotation__

<center> ![](figures/rotation2.png){width=30%} </center>

  We want a set of axises (called Principle Components) that satisfies: \
    -The 1st axis points to the direction where variantion is maximized, and so on\
    -They are orthogonal to each other\
  
  
  It can be shown that the <font color="blue"> eigen vectors of the covariance
  matrix </font> satisfy these conditions, and the eigen vector according to the
  largest eigen value accounts for the most variation.
  
- __Step2: Projection__ (3-dimesion $\rightarrow$ 2-dimension)

<center> ![](figures/pca.png) </center>

##### An example of PCA


```r
deng <- readRDS("data/deng/deng-reads.rds")
my_color1 <- createPalette(6, c("#010101", "#ff0000"), M=1000)
names(my_color1) <- unique(as.character(deng$cell_type1))
my_color2 <- createPalette(10, c("#010101", "#ff0000"), M=1000)
names(my_color2) <- unique(as.character(deng$cell_type2))
deng <- runPCA(deng, ncomponents = 2)
plotPCA(deng, colour_by = "cell_type1") +
  scale_fill_manual(values =  my_color1)
```

<img src="latent-spaces_files/figure-html/unnamed-chunk-1-1.png" width="672" style="display: block; margin: auto;" />

```r
plotPCA(deng, colour_by = "cell_type2") +
  scale_fill_manual(values =  my_color2)
```

<img src="latent-spaces_files/figure-html/unnamed-chunk-1-2.png" width="672" style="display: block; margin: auto;" />

##### Advantages and limits of PCA: 

- __Advantages:__ fast, easy to use and intuitive.

- __Limits:__ 

1. Can lead to local inconsistency, i.e. far away points can become nearest neighbours. 

2. It is a linear projection, like casting a shadow, meaning it can’t capture
non-linear dependencies. For instance, PCA would not be able to “unroll” the
following structure.\ <center> ![](figures/pca_limit.png){width=30%} </center>

#### [GLM-PCA](https://rdrr.io/cran/glmpca/) 
[@collins2002generalization] 
[@townes2019feature]

GLM-PCA is a generalized version of the traditional PCA. 

The traditional PCA implicitly imposes an assumption of Gaussian distribution.
The purpose of GLM-PCA is to loosen this condition to accommodate other
distributions of the exponential family.

Why does PCA assume a Gaussian distribution? \
<font color="#bf812d"> 
 Let $x_1, \dots, x_n \in \mathcal{R}^d$ be the $d$-dimensional data observed. 
 PCA is looking for their projections onto a subspace: $u_1, \dots, u_n$, 
 such that $\sum_{i = 1}^n \Vert x_i - u_i\Vert^2$ is minimized. 
 
 This objective function can be interpretated in two ways: \
Interpretation 1: the variance of the projections/principal components: $\sum_{i}  \Vert u_i \Vert ^2$, if the data is centered at the origin ($\sum_{i} x_i = 0$); \
Interpretation 2: Each point $x_i$ is thought of as a random draw from a probability distribution centered at $u_i$.
If we take this probability as a unit Gaussian, that is $x_i \sim N(u_i, 1)$, then the likelihood is 
$\prod_{i = 1}^n \exp (- \Vert x_i - u_i\Vert^2)$, and the negative log likelihood is exactly the objective function.
 </font>

This assumption is often inappropriate for non-Gaussian distributed data, for
example discrete data.

Therefore, GLM-PCA generalizes the Gaussian likelihood into a likelihood of any
exponential-family distribution, and applies appropriate link functions to
$u_i$'s in the same as a GLM does to non-Gaussian responses.

The following example compares GLM-PCA with Poisson marginals to the
traditional PCA, which is identical to the result from ```plotPCA```.


```r
## GLM-PCA
Y <- assay(deng, "counts")
Y <- Y[rowSums(Y) > 0, ]
system.time(res1 <- glmpca(Y, L=2, fam="poi", verbose=TRUE))
```

```
##    user  system elapsed 
##  84.147  22.275 106.448
```

```r
pd1 <- data.frame(res1$factors, dimreduce="glmpca-poisson", clust = factor(deng$cell_type2))
## traditional PCA
pd2 <- data.frame(reducedDim(deng, "PCA"), dimreduce="runPCA", clust = factor(deng$cell_type2))
colnames(pd2) <- colnames(pd1)
## plot
pd <- rbind(pd1, pd2)
ggplot(pd, aes(x = dim1, y = dim2, colour = clust)) +
  geom_point(size=2) +
  facet_wrap(~dimreduce, scales="free", nrow=3) +
  scale_color_manual(values =  my_color2) +
  theme_bw()
```

<img src="latent-spaces_files/figure-html/glmpca-1.png" width="672" style="display: block; margin: auto;" />

Let us compare GLM-PCA and standard PCA (using normalized log-counts data) on
the Tung data, before cells have been QC'd.

<img src="latent-spaces_files/figure-html/unnamed-chunk-2-1.png" width="672" style="display: block; margin: auto;" />

Repeat these plots with the QC'd Tung data.

<img src="latent-spaces_files/figure-html/unnamed-chunk-3-1.png" width="672" style="display: block; margin: auto;" />

### tSNE: t-Distributed Stochastic Neighbor Embedding

t-SNE [@maaten2008visualizing] is an advanced version of the original SNE algorithm. [@hinton2003stochastic]

#### Motivation 

The weakness of PCA is the motivation behind the SNE algorithm.

1. PCA focuses on global covariance structrue, which lead to local inconsistency. \
SNE aims to preserve local strucutrue, or preserving the relationships
among data points (ie. similar points remain similar; distinct points remain distinct).

2. Unlike PCA, SNE is not limited to linear projections, which makes it suited to all sorts of datasets, including the swiss-roll data we have seen above.

3. t-SNE solves the crowding issue of the original SNE. 

#### original SNE 

SNE minimizes the divergence between two distributions: a distribution that
measures pairwise similarities of the input objects and a distribution that
measures pairwise similarities of the corresponding low-dimensional points in
the embedding.


- __Goal:__ preserve neighbourhoods.

- __Soft neighbourhood:__

   For each data point $x_i$, 
   the $i\rightarrow j$ probability is the probability that
point $x_i$ chooses $x_j$ as its neighbour:
   $p_{j|i} \propto \exp(-\Vert x_i - x_j \Vert^2/2\delta^2)$.\
   (This can be thought of as the probability of $x_j$ in $N(x_i, \delta)$)
   
   - $\Vert x_i - x_j \Vert^2$ is the Euclidean distance: \
   The closer $x_i$ and $x_j$ are, the larger $p_{j|i}$ is. 
   
   - $\delta^2$ denotes the vairance, it sets the size of the neighbourhood.  
        &nbsp;    Very low $\Rightarrow$ all the probability is in the nearest neighbour\
        &nbsp;    Very high $\Rightarrow$  uniform weights \
     <center> ![](figures/Gaussian.png){width=60%} </center>
     We generally want $\delta^2$  to be small for points in densely populated areas and large for sparse areas, so that the number of neighbours of all data points are roughly the same. 
     It is computed with a user specified parameter (__perplexity__) which indicates the effective number of neighbours for a data point. 
    
- __Similarity matrix__\
  
  Collect $p_{j|i}$ for all data points into a matrix, then this matrix preserves the key information  of the local neighbourhood structure. 
  
  
- __How SNE works:__ \

  a. Given high-dimensional data $X = \{x_1, \dots, x_n \in \mathcal{R}^d \}$, 
    obtain the similarity matrix $P$; 
  
  b. Let $Y =\{y_1, \dots, y_n \in \mathcal{R}^2\}$ be a 2-dimensional data, the coordinates for visualization. Obtain a smilarity matrix of $Y$, denoted  as $Q$, in the same way as $X$, except that $\delta^2$ is fixed at 1/2.
  
  c. Look for $Y$ such that $Q$ is as similar to $P$ as possible.\
    Measurement of how similar two distributions is: __Kullback-Leibler divergence__ \
    (The definition of this cost function and the optimization procedure are out of the scope of this course)
  
#### t-SNE 

The motivation of t-SNE is to solve one of the main issues of SNE, the crowding problem.
  
- __Crowding problem:__\
In high dimension we have more room, points can have a lot of different
neighbours that are far apart from each other. 
But in low dimensions, we don’t have enough room to
accommodate all neighbours.
Forexample, in 2D a point can have a few neighbors at distance one all far from each
other - what happens when we embed in 1D?
<center>![](figures/crowding.png){width=60%}</center>


- __Solution:__\
Change the distribution of the low-dimensional data $Q$ into a student-t distribution. 

<center>![](figures/t.png){width=50%} </center>

Recall that SNE is trying to minimize the dissimilarity of $P$ and $Q$, 
and $P$ has a Gaussian distribution. 
So for a pair of points ($x_i$ and $x_j$ in high-dimension, $y_i$ and $y_j$ in low-dimension) to reach the same probability, the distance between $y_i$ and $y_j$ would be much larger (i.e. much farther apart).

#### __Example of t-SNE:__\ 


```r
muraro <- readRDS("data/pancreas/muraro.rds")
tmp <- runTSNE(muraro, perplexity = 3)
plotTSNE(tmp, colour_by = "cell_type1")
tmp <- runTSNE(muraro, perplexity = 50)
plotTSNE(tmp, colour_by = "cell_type1")
```

<img src="latent-spaces_files/figure-html/tsne-1.png" width="672" style="display: block; margin: auto;" /><img src="latent-spaces_files/figure-html/tsne-2.png" width="672" style="display: block; margin: auto;" />


#### __Limits of t-SNE:__\
1. Not a convex problem, i.e. the cost function has multiple local minima.\
2. Non-deterministic. 
3. Require specification of very important parameters, e.g. perplexity.\
4. Coordinates after embedding have no meaning. 
Therefore can merely be used for visualization.\

(See [here](https://distill.pub/2016/misread-tsne/) for more pitfalls of using t-SNE.)

### Manifold methods

#### UMAP: Uniform Manifold Approximation and Projection [@mcinnes2018umap]

##### __Advantages of UMAP over t-SNE:__

  - faster\
  - deterministic\
  - better at preserving clusters\
  
##### High level description

1. Construct a topological presentation of the high-dimensional data (in this case a weighted $k$-NN graph)\
2. Given a low-dimensional data, construct a graph in the similar way \
3. Minimize the dissimilarity between the twp graphs. \
   (Look for the low-dimensional data whose graph is the closest to that of the high-dimensional data)
   
##### Some details 

- __How the weighted graph is built?__

i) Obtain dissimilarity from the input distance:\
  For each data point $x_i$, find its $k$ nearest neighbours: $x_{i_1}, \dots, x_{i_k}$.\
  Let $d(x_i, x_{i_j})$ be the input or original distance between $x_i$ and $x_{i_j}$, 
  and $\rho_i = \min[d(x_i, x_{i_j}); 1 \leq j \leq k]$ be the distance between $x_i$ and its nearest neighbour.  
  Then the dissimilarity between $x_i$ and $x_{i_j}$ is measured simply by subtracting the original distance by $\rho_i$: 
  $\tilde{d}(x_i, x_{i_j}) = d(x_i, x_{i_j}) - \rho_i$. \
  
ii) Tranform dissimilarity to similarity:\
    $s(x_i, x_{i_j}) = \exp[-\tilde{d}(x_i, x_{i_j})] - c_i$, where $c_i$ is a scale factor to ensure $\sum_{j = 1}^k s(x_i, x_{i_j})$ is a constant for all $i$.\
    Similarity itself can serve as edge weights, but this similarity is not symmetrical, i.e.  $s(x_i, x_{i_j}) \neq  s(x_{i_j}, x_i)$. 
    To be able to project this onto an undirected graph, we need to solve the disagreement between  $s(x_i, x_{i_j})$ and $s(x_{i_j}, x_i)$.

iii) Obtain weights:\
  $w(x_i, x_{i_j}) = s(x_i, x_{i_j}) + s(x_{i_j}, x_i) - s(x_i, x_{i_j}) * s(x_{i_j}, x_i)$\
  (Interpretation: $P(A \cup B ) = P(A) + P(B) - P(A)P(B)$ if $A$ and $B$ are independent)
 

- __How the dissimilarity between graphs are measured?__ Cross entropy 

##### Example of UMAP


```r
muraro <- runUMAP(muraro)
plotUMAP(muraro, colour_by="cell_type1")
```

<img src="latent-spaces_files/figure-html/umap-1.png" width="672" style="display: block; margin: auto;" />
  
  

#### PHATE [@moon2017phate]

##### Sketch of algorithm 

- __The simpliest description of PHATE:__\
__Step1.__ Create a dissimilarity matrix of the original data\
__Step2.__   Feed the dissimilarity matrix to nonmetric MDS\
(__MDS__: Multi-Dimension Scaling is a classical dimensionality reduction approach, that takes an input of distance matrix, and aims at preserving pairwise distances in the low dimensional space. 
When the input distance matrix is Euclidean distance, MDS produces the same result as PCA. 
Nonmetric MDS generalize the input as a dissimilarity matrix, rather than just distance.)


 
- __Details of step1 in PHATE__ \
__Step1-1. Markov transition matrix__  \
           - What is __similar__ with SNE: \
             Recall that in the original SNE algorithm, there is a similarity matrix with entry 
             $p_{i|j}$ that is interpreted as the probability that 
             point $x_i$ chooses $x_j$ as its neighbour:
             $p_{j|i} \propto \exp(-\Vert x_i - x_j \Vert^2/2\delta^2)$.\
             PHATE is doing the same, except that we can interpret it differently: \
             i. We can think $p_{j|i}$ as a Gaussian kernel, where $\epsilon \triangleq 2\delta^2$ is the bandwidth: $p_{j|i} \triangleq K_\epsilon(x_i, x_j )$. 
                Similar to SNE, PHATE also define $\epsilon$ as the $k$-NN distance of each data point, so that it is smaller in dense area and larger in sparse area. 
                The $k$ is a user-specified tuning parameter, similar to perplexity in SNE. \
             ii. We can think of the similarity matrix as a transition matrix, where $p_{j|i}$ represents the probability of 
             jumping from state $i$ to state $j$ in a single step. \
             - What is __different__:\
             i. PHATE generalize $K_\epsilon(x_i, x_j)$ to $\exp \left(- \Vert x_i - x_j \Vert^\alpha /\epsilon(x_i)^\alpha\right)$, where the original 
                Gaussian kernel is the special case when $\alpha = 2$.\
                The motivation is that if the data is very sparse in some regions, then the bandwith $\epsilon$ with be very large and the kernel will become flat and lose the local information. 
                By letting $\alpha > 2$, we prevent this to happen, although $\alpha$ needs to be provided by the user. \  \
             ii. Note that the kernels are not symmetrical now, that is  $K_\epsilon(x_i, x_j) \neq K_\epsilon(x_j, x_i)$. 
                 So we make it symmetrical by taking an average of the two. \  \
__Step1-2. Smoothing__ \
              - $P$ is the transition matrix where $p_{i, j}$ represents the probability of jumping from state $i$ to state $j$ in a single step.\
              - Denote $\delta_x$ as a row vector of length $n$ (the number of data points), where only the entry corresponding to $x$ is 1 and zero everywhere else. \
                Then $p_x = \delta_x P$ is the probability distribution of the data points starting from $x$ after one step, 
<center>![](figures/transP.png){width=50%}</center>
and $p_x^t = \delta_x P^t$ is the probability distribution of the data points after $t$ steps.\
In general, the more steps we take, the more data points will have positive probabilities. \
One way to think about this, is the larger $t$ is, the more global information and the less local information is encoded.\
In the extreme case, if we take infinity steps, $p_x^\infty$ will be the same for all $x$'s, i.e. 
the probability distribution is going to be the same regardless of where we start, in this case, the local information is completely lost.\
An appropriately chosen $t$ is crucial for the balance between local and global information in the embedding. 
(See the [original paper]("https://www.biorxiv.org/content/10.1101/120378v1") for details of choosing $t$)\
__Step1-3. Distance measurement__ \
Instead of measuring directly the Eudlicean distance between data points, say $x_i$ and $x_j$, 
PHATE measures the distance between probability distributions $p_{x_i}^t$ and $p_{x_j}^t$:\
$D^t(x_i, x_j) = \Vert \log(p_{x_i}^t) - \log(p_{x_j}^t) \Vert^2$

##### Example of PHATE 


```r
library(phateR)
deng_phate <- phate(t(assay(deng, "logcounts")))
dt <- data.frame(deng_phate$embedding, clust = deng$cell_type1)
palette(rainbow(10))
ggplot(dt, aes(x=PHATE1, y=PHATE2, color=clust)) +
  geom_point()
```
<center> ![](figures/phate.png) </center>


## Matrix factorization and factor analysis

__The key concept of factor analysis__: The original, observed variables are
correlated because they are all associated with some unobservable variables,
the __latent factors__.

It looks similar to PCA, but instead of dimensionality reduction, factor analysis 
focuses on studying the latent factors. 

The variance of an observed variable can be splitted into two parts: \
- Common variance: the part of variance that is explained by latent factors; \
- Unique variance: the part that is specific to only one variable, usually considered as an error component or __residual__.

The __factor loadings__ or weights indicate how much each latent factor is affecting the observed features. 

<center> ![](figures/FA.png){width=60%} </center>

### [Slalom](https://bioconductor.org/packages/release/bioc/html/slalom.html): Interpretable latent spaces

Highlight of Slalom: [@buettner2017f]

- It incorporates prior information to help the model estimation; 

- It learns whatever not provided by prior knowledge in the model training process; 

- It enforces sparsity in the weight matrix. 

#### Methodology

__Matrix expression of factor analysis:__

<center>![](figures/FA_matrix.png){width=80%}      </center>

__How prior knowledge affects the model:__

<center>![](figures/slalom_anno.png)     </center>

- $I_{g, k}$: (observed) Indicator of whether a gene $g$ is annotated to a given pathway or factor $k$;\
- $z_{g, k}$: (latent) Indicator of whether factor $k$ has a regulatory effect on gene $g$;\
- $w_{g, k}$: (estimated) weights.

__grey arrow__:
$$ P(I_{g, k}\vert z_{g, k}) = \begin{cases}
\text{Bernoulli}(p_1),    \text{if } z_{g, k} = 1\\
\text{Bernoulli}(p_2),     \text{if } z_{g, k} = 0\\
\end{cases}$$

__green arrow__: 
$$ P(w_{g, k}\vert z_{g, k}) = \begin{cases}
N(w_{g, k}, 1/\alpha),    \text{ if } z_{g, k} = 1\\
\delta_0(w_{g, k}),     \text{   if } z_{g, k} = 0\\
\end{cases}$$

<center>![](figures//slab_spike.png)</center>

We only look at the part of the __likelihood__ that is relavant to this part: 
$\prod_{g} \prod_{k}P(I_{g, k}, w_{g, k}, z_{g, k})$, \
where $P(I_{g, k}, w_{g, k}, z_{g, k}) = P(I_{g, k}, w_{g, k}| z_{g, k})P(z_{g,k})
= P( I_{g, k}| z_{g, k})P( w_{g, k}| z_{g, k})P(z_{g,k})$.
Since we do not know anything about $z_{g,k}$, it is assumed as Bernoulli(1/2).

#### Example
First, get a geneset in a `GeneSetCollection` object.

```r
gmtfile <- system.file("extdata", "reactome_subset.gmt", package = "slalom")
genesets <- GSEABase::getGmt(gmtfile)
```

Then we create an `Rcpp_SlalomModel` object containing the input data and genesets (and subsequent results) for the model. 

```r
model_deng <- newSlalomModel(deng, genesets, n_hidden = 5, min_genes = 10)
```

```
## 29 annotated factors retained;  1 annotated factors dropped.
## 1072  genes retained for analysis.
```

Initialize the model:

```r
model_deng <- initSlalom(model_deng, seed = 100)
```

Fit/train the model:

```r
model_deng <- trainSlalom(model_deng, nIterations = 1000, seed = 100, tolerance = 0.001)
```

```
## pre-training model for faster convergence
## iteration 0
## Model not converged after 50 iterations.
## iteration 0
## Model not converged after 50 iterations.
## iteration 0
## Switched off factor 29
## Switched off factor 20
## Switched off factor 32
## Switched off factor 28
## Switched off factor 13
## Switched off factor 27
## Switched off factor 10
## iteration 100
## Switched off factor 22
## iteration 200
## iteration 300
## iteration 400
## iteration 500
## iteration 600
## iteration 700
## Model converged after 701 iterations.
```

View results:\
The `plotRelevance` function displays the most relevant terms (factors/pathways) ranked by relevance, showing gene set size and the number of genes gained/lost as active in the pathway as learnt by the model.

```r
plotRelevance(model_deng) + theme_classic(base_size = 8)
```

<img src="latent-spaces_files/figure-html/unnamed-chunk-8-1.png" width="1344" style="display: block; margin: auto;" />

```
## NULL
```
The `plotTerms` function shows the relevance of all terms in the model, enabling the identification of the most important pathways in the context of all that were included in the model.

```r
plotTerms(model_deng)
```

<img src="latent-spaces_files/figure-html/unnamed-chunk-9-1.png" width="672" style="display: block; margin: auto;" />

## Autoencoders 
[@kingma2013auto]

<center>![](figures/vae.jpg){width=80%}</center>

### Background and some notations

- __Data__: $X$
- __Latent variables:__ $Z$\
Something that is not directly observable but is assumed to have an impact on the observed variables.

- __Goal:__ We believe $X$ can be generated from $Z$ (with some trasformation), and want to sample more data from $Z$ that resembles $X$. 
So we want the to find the parameters $\theta$ such that the probability to generate $X$ from the distribution of $Z$: $P(X) = \int P(X|z; \theta) P(z) dz$ is maximized. 


- __How do we define $Z$?__\
  - __The simplest idea:__ $Z \sim N(0, 1)$. 
 It is not impossible, because "any distribution in d dimensions can be generated by taking a set of d variables that are normally distributed and mapping them through a sufficiently complicated function. "\
  -__A better idea:__ 
 For most of $z$, $P(X|z; \theta)$ will be close to zero, meaning it contribute almost nothing to the estimate of $P(X)$. Thus, we want to sample only those values of $Z$ that are likely to produce $X$.
 Denote this distribution of $Z$ as $Q(Z|X)$ (it is infered and therefore depend on $X$).\
 __Advantage:__ There will be a lot less possible values of $Z$ under $Q$ compared to random sampling, therefore, it will be easier to compute $E_{Z \sim Q} P(X|Z)$. 

 
### Objective 

$$ \log P(X) - KL[Q(Z|X)\Vert P(Z|X)] = E_{Z\sim Q}[\log P(X|Z)] - KL[Q(Z|X)\Vert P(Z)]$$
 We can get this equation by starting from the definition of Kullback-Leibler divergence, combined with the Bayesian formula and a little algebra. (Not showing details here) 
 
 - LHS: what we want to maximize: 
        Generation loss: how likely the generated samples resembles $X$ __-__
        an error term which measures how much information is lost when using $Q$ to represent $P$, 
        it becomes small if $Q$ is high-capacity. 
        (_A loose explanation of model capacity: Roughly speaking, the capacity of a model describes how complex a relationship it can model. You could expect a model with higher capacity to be able to model more relationships between more variables than a model with a lower capacity._)\
- RHS: what we can maximize through stochastic gradient descent. 
        









