---
output: html_document
---


## Read alignment and gene expression quantification


Now we have trimmed our reads and established that they are of good quality, we
would like to map them to a reference genome. This process is known as
alignment. Some form of alignment is generally required if we want to quantify
gene expression or find genes which are differentially expressed between
samples.

Many tools have been developed for read alignment. STAR [@dobin_2013] is one of
most popularly used tools in RNA-seq read alignment. There are a bunch of
alignment and gene expression quantification tools that are designed
specifically for single-cell RNA-seq data too. Depending on your single-cell
RNA-seq protocols used and the datasets generated, you can go with the following
two workflows presented here for full-length or tag-based datasets.

Thus, today we will focus on two alignment tools: STAR and Kallisto-BUStools,
and we will discuss other available tools at the end of this chapter.


## Full-length transcript datasets 

If your single-cell RNA-seq dataset is from plate-based protocol like
Smart-seq2, then your dataset can be aligned and quantifified just like a bulk
RNA-seq datasest. Each cell has a proper pair (if it's paired-end sequncing) of
fastq files (there are no CB/UMI tags in the reads). STAR is a good choice for
alignment (other good choices could be Subread or Hisat2).


### Using STAR to align reads

STAR tries to find the longest possible sequence which matches one or more
sequences in the reference genome. For example, in the figure below, we have a
read (blue) which spans two exons and an alternative splicing junction (purple).
STAR finds that the first part of the read is the same as the sequence of the
first exon, whilst the second part of the read matches the sequence in the
second exon. Because STAR is able to recognise splicing events in this way, it
is described as a 'splice aware' aligner.


<div class="figure" style="text-align: center">
<img src="figures/star_map.jpg" alt=" Diagram of how STAR performs alignments, taken from Dobin et al." width="80%" />
<p class="caption">(\#fig:unnamed-chunk-2) Diagram of how STAR performs alignments, taken from Dobin et al.</p>
</div>

Usually STAR aligns reads to a reference genome, potentially allowing it to
detect novel splicing events or chromosomal rearrangements.

### Expression quantification

Now you have your aligned reads in a `.bam` file for your single cells. The next
step is to quantify the expression level of each gene per cell. We can use one
of the tools which has been developed for bulk RNA-seq data, e.g.
[HT-seq](http://www-huber.embl.de/users/anders/HTSeq/) or
[FeatureCounts](http://bioinf.wehi.edu.au/featureCounts/) which do 'simple'
counting of reads overlapping with genomic features.


Here we demostrate an example with `featureCounts`, that counts mapped reads for
genomic features such as genes, exons, promoter, gene bodies, genomic bins and
chromosomal locations.

```
# include multimapping
<featureCounts_path>/featureCounts -O -M -Q 30 -p -a hg_annotations.gtf -o outputfile ERR522959.bam

# exclude multimapping
<featureCounts_path>/featureCounts -Q 30 -p -a hg_annotations.gtf -o outputfile ERR522959.bam

```

Then you will have your read counts gene expression matrix that's ready for
downstream analysis.


## Tag-based datasets

If your dataset is tag-based, for example 10X dataset, then you typically have
sequences in R1 that entirely encode for read identities such as Cell Barcode
and UMI tags. In most cases, all your cell reads (for 1k-10K cells) are in one
set of fastq files. Instead of trying to demultiplex all cells into separate
fastqs then do alignment and quantification, we can use tools that take care of
this for you. In the following steps, we use Kallisto with bustools for
generating the gene expression quantification matrix for your tag-based
datasets.

### Cellranger count

If you work with 10X dataset, `cellranger count` pipeline may just work well for
you. It comes with cellranger software suite with convenient features for 10X
datasets. It takes the fastqs of a sample, and uses STAR to align all cells'
reads. It also includes reads filtering, barcode counting, and UMI counting. The
output of this pipeline includes the `aligned.bam` file and the quantified gene
expression matrix in both filtered and raw format. In V3, it has adopted the
`EmptyDroplet` method (Lun et al., 2018), an algorithm that tries distinguish
true cell barcodes from barcodes associated with droplets that did not contain a
cell (i.e. empty droplets). (More details in `Section 5.9` ). The filtered gene
expression matrix by `cellranger count` V3 only includes the true cell barcodes
determined by `EmptyDroplet` method.

### Kallisto/bustools and pseudo-alignment

STAR is a reads aligner, whereas Kallisto is a pseudo-aligner [@bray_2016]. The
main difference between aligners and pseudo-aligners is that whereas aligners
map reads to a reference, pseudo-aligners map k-mers to a reference.

### What is a k-mer?

A k-mer is a sequence of length k derived from a read. For example, imagine we
have a read with the sequence ATCCCGGGTTAT and we want to make 7-mers from it.
To do this, we would find the first 7-mer by counting the first seven bases of
the read. We would find the second 7-mer by moving one base along, then counting
the next seven bases. Below shows all the 7-mers that could be derived from our
read:

``` 
ATCCCGGGTTAT

ATCCCGG
 TCCCGGG
  CCCGGGT
   CCGGGTT
    CGGGTTA
     GGGTTAT
```

### Why map k-mers rather than reads?

There are two main reasons:

1. Pseudo-aligners use k-mers and a computational trick to make pseudo-alignment
   much faster than traditional aligners. If you are interested in how this is
   acheived, see [@bray_2016] for details.

2. Under some circumstances, pseudo-aligners may be able to cope better with
   sequencing errors than traditional aligners. For example, imagine there was a
   sequencing error in the first base of the read above and the A was actually a
   T. This would impact on the pseudo-aligners ability to map the first 7-mer
   but none of the following 7-mers.

### Kallisto's pseudo mode

Kallisto has a specially designed mode for pseudo-aligning reads from
single-cell RNA-seq experiments. Unlike STAR, Kallisto psuedo-aligns to a
reference transcriptome rather than a reference genome. This means Kallisto maps
reads to splice isoforms rather than genes. Mapping reads to isoforms rather
than genes is especially challenging for single-cell RNA-seq for the following
reasons:

 * Single-cell RNA-seq is lower coverage than bulk RNA-seq, meaning the total
   amount of information available from reads is reduced.
 * Many single-cell RNA-seq protocols have 3' coverage bias, meaning if two
   isoforms differ only at their 5' end, it might not be possible to work out
   which isoform the read came from.
 * Some single-cell RNA-seq protocols have short read lengths, which can also
   mean it is not possible to work out which isoform the read came from.

Kallisto's pseudo mode takes a slightly different approach to pseudo-alignment.
Instead of aligning to isoforms, Kallisto aligns to equivalence classes.
Essentially, this means if a read maps to multiple isoforms, Kallisto records
the read as mapping to an equivalence class containing all the isoforms it maps
to. Figure 2 shows a diagram which helps explain this.

<div class="figure" style="text-align: center">
<img src="figures/Kallisto.jpg" alt=" Overview of kallisto, The input consists of a reference transcriptome and reads from an RNA-seq experiment. (a) An example of a read (in black) and three overlapping transcripts with exonic regions as shown. (b) An index is constructed by creating the transcriptome de Bruijn Graph (T-DBG) where nodes (v1, v2, v3, … ) are k-mers, each transcript corresponds to a colored path as shown and the path cover of the transcriptome induces a k-compatibility class for each k-mer. (c) Conceptually, the k-mers of a read are hashed (black nodes) to find the k-compatibility class of a read. (d) Skipping (black dashed lines) uses the information stored in the T-DBG to skip k-mers that are redundant because they have the same k-compatibility class. (e) The k-compatibility class of the read is determined by taking the intersection of the k-compatibility classes of its constituent k-mers. Taken from Bray et al (2016)." width="80%" />
<p class="caption">(\#fig:unnamed-chunk-3) Overview of kallisto, The input consists of a reference transcriptome and reads from an RNA-seq experiment. (a) An example of a read (in black) and three overlapping transcripts with exonic regions as shown. (b) An index is constructed by creating the transcriptome de Bruijn Graph (T-DBG) where nodes (v1, v2, v3, … ) are k-mers, each transcript corresponds to a colored path as shown and the path cover of the transcriptome induces a k-compatibility class for each k-mer. (c) Conceptually, the k-mers of a read are hashed (black nodes) to find the k-compatibility class of a read. (d) Skipping (black dashed lines) uses the information stored in the T-DBG to skip k-mers that are redundant because they have the same k-compatibility class. (e) The k-compatibility class of the read is determined by taking the intersection of the k-compatibility classes of its constituent k-mers. Taken from Bray et al (2016).</p>
</div>

<div class="figure" style="text-align: center">
<img src="figures/EC_kallisto.png" alt=" A diagram explaining Kallisto's Equivalence Classes, taken from Ntranos et al." width="80%" />
<p class="caption">(\#fig:unnamed-chunk-4) A diagram explaining Kallisto's Equivalence Classes, taken from Ntranos et al.</p>
</div>

__Note__ Instead of using gene or isoform expression estimates in downstream
analysis such as clustering, equivalence class counts can be used instead, in
this course, we focus on using gene level estimation.

### Running kallisto pseudo-alignment and BUStools

Today, we will talk about doing single-cell pseudo-alignemnt and gene level
quantification with `Kallisto|BUStools`. See
https://pachterlab.github.io/kallisto/manual for details.

As for STAR, you will need to produce an index for Kallisto before the
pseudo-alignment step.

Use the below command to produce the Kallisto index. Use the Kallisto manual
(https://pachterlab.github.io/kallisto/manual) to work out what the options do
in this command.


```bash
mkdir indices/Kallisto
kallisto index -i indices/Kallisto/GRCm38.idx Share/mouse/Ensembl.GRCm38.96/Mus_musculus.GRCm38.cdna.all.fa.gz
```

In this step, an index is constructed by creating the transcriptome de Bruijn
Graph (T-DBG).

#### BUS format 

`BUS` is a binary file format designed for UMI-tagged single-cell datasets with
pseudo-aligned reads labelled with CB and UMI tags.

<div class="figure" style="text-align: center">
<img src="figures/bus_format.png" alt=" BUS format, taken from Melsted,Páll et al." width="80%" />
<p class="caption">(\#fig:unnamed-chunk-6) BUS format, taken from Melsted,Páll et al.</p>
</div>

We do `kallisto bus` on the fastqs of single cells to generate the BUS file and
then use `BUStools` on the generated bus files to get a gene level
quantification.

Check the list of technologies supported by Kallisto BUStools by `kallisto bus
-l` Use the below command to perform pseudo-alignment and generate bus files for
single-cell sequencing data. `-x` argument specifies the technology.

```
List of supported single-cell technologies

short name       description
----------       -----------
10xv1            10x version 1 chemistry
10xv2            10x version 2 chemistry
10xv3            10x version 3 chemistry
CELSeq           CEL-Seq
CELSeq2          CEL-Seq version 2
DropSeq          DropSeq
inDrops          inDrops
SCRBSeq          SCRB-Seq
SureCell         SureCell for ddSEQ
```




```bash
mkdir results/Kallisto

kallisto bus -i indices/Kallisto/GRCm38.idx -o  results/Kallisto/output_bus -x '10xv2' -t 4 \
  SI-GA-G1/W11_S1_L001_R1_001.fastq.gz SI-GA-G1/W11_S1_L001_R2_001.fastq.gz \
  SI-GA-G1/W11_S1_L002_R1_001.fastq.gz SI-GA-G1/W11_S1_L002_R2_001.fastq.gz
```

See https://pachterlab.github.io/kallisto/manual for instructions on creating bus files.

### Understanding the Output of Kallisto BUS Pseudo-Alignment

The command above should produce 4 files - matrix.ec, transcripts.txt,
run_info.json and output.bus

* `transcripts.txt` contains a list of transcript, in the same order as in the
  transcriptome fasta file.
* `matrix.ec` contains information about the equivalence classes used. The first
  number in each row is the equivalence class ID. The second number(s)
  correspond to the transcript ID(s) in that equivalence class. For example "10
  1,2,3" would mean that equivalence class 10 contains transcript IDs 1,2 and 3.
  The ID numbers correspond to the order that the transcripts appear in
  transcripts.txt. Zero indexing is used, meaning transcript IDs 1,2 and 3
  correspond to the second, third and fourth transcripts in transcripts.txt.
* `output.bus` contains the binary formated Cell Barcode and UMI tags and Sets of
  equivalent classes of transcripts obtained by pseudoalignment.(The fourth
  column is count of reads with this barcode, UMI, and equivalence class
  combination, which is ignored as one UMI should stand for one molecule.)
* `run_info.json` contains information about how Kallisto was executed and can be ignored.


### Running Bustools

Inputs: 

- `transcripts_to_genes.tsv`: 
  a tab delimited file of a specific format: No headers, first column is transcript ID, and second column is the corresponding gene ID. Transcript IDs must be in the same order as in the kallisto index.
- *barcode whitelist*: 
  A whitelist that contains all the barcodes known to be present in the kit is provided by 10x and comes with CellRanger.
  
First, bustools runs barcode error correction on the bus file. Then, the
corrected bus file is sorted by barcode, UMI, and equivalence classes. After
that the UMIs are counted and the counts are collapsed to the gene level.
  

```bash
mkdir ./output/out_bustools/genecount ./tmp

bustools correct -w ./data/whitelist_v2.txt -p ./output/out_bustools/output.bus | \
bustools sort -T tmp/ -t 4 -p - | \
bustools count -o ./output/out_bustools/genecount/genes -g ./output/tr2g_hgmm.tsv \
-e ./output/out_bustools/matrix.ec -t ./output/out_bustools/transcripts.txt --genecounts -
```

The output includes:

```
genes.barcodes.txt
genes.genes.txt
genes.mtx

```

### Other alignment and quantification tools available
  
- Alevin
  
  [Alevin](https://salmon.readthedocs.io/en/latest/alevin.html) is a tool for
  10X and Drop-seq data that comes with `Salmon` which is also a
  'pseudo-aligner' for transcriptome quantification. `Salmon` is conceptually
  simiarly to Kallisto but uses different models for parameter estimation
  and account for sequence (3' 5'-end and Fragment GC) bias correction.

- STARsolo
  
  STARsolo is integrated with STAR. It does mapping, demultiplexing and gene
  quantiﬁcation for droplet-based single-cell RNA-seq (eg. 10X genomics). It
  follows a similar logic as `Cellranger count` pipeline which does error
  correction, UMI deduplication and then quantify expression per gene for each
  cell by counting reads with different UMIs mapped per gene. STARsolo is
  potentially ten times faster than `Cellranger count`.

If you are interested, here is a paper by Páll et al that compares performance
of workflows in single-cell RNA-seq preprocessing.
(https://www.biorxiv.org/content/10.1101/673285v2.full).

### Summary

- _Full-transcripts dataset:_

  STAR -> featureCounts

- _Tag-based dataset:_

  Kallisto bus -> Bustools 


## Practise 

### Using STAR

One issue with STAR is that it needs a lot of RAM, especially if your
reference genome is large (eg. mouse and human). To speed up our analysis today,
we will use STAR to align reads to a reference genome.

Two steps are required to perform STAR alignment. In the first step, the user
provides STAR with reference genome sequences (FASTA) and annotations (GTF),
which STAR uses to create a genome index. In the second step, STAR maps the
user's reads data to the genome index.

Let's create the index now. You can obtain genomes for many model organisms from
Ensembl (https://www.ensembl.org/info/data/ftp/index.html**.

**Task 1:** Execute the commands below to create the index:


```bash
mkdir indices

mkdir indices/STAR

STAR --runThreadN 4 --runMode genomeGenerate --genomeDir indices/STAR --genomeFastaFiles Share/hg19.fa --sjdbGTFfile Share/hg_annotations.gtf
```

**Task 2:** What does each of the options we used do? Hint: Use the STAR manual
to help you (https://github.com/alexdobin/STAR/blob/master/doc/STARmanual.pdf**

**Task 3:** How would the command we used in Task 1 be different if we were
aligning to the genome rather than the transcriptome?

Now that we have created the index, we can perform the mapping step.

**Task 4:** Try to work out what command you should use to map our trimmed reads
(from ERR522959** to the index you created. Use the STAR manual to help you. One
you think you know the answer, check whether it matches the solution in the next
section and execute the alignment.

**Task 5:** Try to understand the output of your alignment. Talk to one of the
instructors if you need help!

### Solution for STAR Alignment

You can use the folowing commands to perform the mapping step:


```bash
mkdir results
mkdir results/STAR

STAR --runThreadN 4 --genomeDir indices/STAR --readFilesIn Share/ERR522959_1.fastq Share/ERR522959_2.fastq \
  --outFileNamePrefix results/STAR/ERR522959 
```


