---
output: html_document
---



## Batch effects

### Introduction

In the previous chapter we normalized for library size, effectively removing it
as a confounder. Now we will consider removing other less well defined
confounders from our data. Technical confounders (aka batch effects) can arise
from difference in reagents, isolation methods, the lab/experimenter who
performed the experiment, even which day/time the experiment was performed.
Accounting for technical confounders, and batch effects particularly, is a large
topic that also involves principles of experimental design. Here we address
approaches that can be taken to account for confounders when the experimental
design is appropriate.

Fundamentally, accounting for technical confounders involves identifying and,
ideally, removing sources of variation in the expression data that are not
related to (i.e. are confounding) the biological signal of interest. Various
approaches exist, some of which use spike-in or housekeeping genes, and some of
which use endogenous genes.

#### Advantages and disadvantages of using spike-ins to remove confounders

The use of spike-ins as control genes is conceptually appealing, since (ideally)
the same amount of ERCC (or other) spike-in would be added to each cell in our
experiment. In principle, all the variability we observe for these ``genes'' is
due to technical noise; whereas endogenous genes are affected by both technical
noise and biological variability. Technical noise can be removed by fitting a
model to the spike-ins and "substracting" this from the endogenous genes. There
are several methods available based on this premise (eg.
[BASiCS](https://github.com/catavallejos/BASiCS),
[scLVM](https://github.com/PMBio/scLVM),
[RUVg](http://bioconductor.org/packages/release/bioc/html/RUVSeq.html)); each
using different noise models and different fitting procedures. Alternatively,
one can identify genes which exhibit significant variation beyond technical
noise (eg. Distance to median, [Highly variable
genes](http://www.nature.com/nmeth/journal/v10/n11/full/nmeth.2645.html)).


Unfortunately, there are major issues with the use of spike-ins for
normalisation that limit their utility in practice. Perhaps surprisingly, their
variability can, for various reasons, actually be *higher* than that of
endogenous genes. One key reason for the difficulty of their use in practice is
the need to pipette miniscule volumes of spike-in solution into The most popular
set of spike-ins, namely ERCCs, are derived from bacterial sequences, which
raises concerns that their base content and structure diverges to far from gene
structure in other biological systems of interest (e.g. mammalian genes) to be
reliable for normalisation. Even in the best-case scenarios, spike-ins are
limited to use on plate-based platforms; they are fundamentally incompatible
with droplet-based platforms.

Given the issues with using spike-ins, better results can often be obtained by
using endogenous genes instead. Given their limited availability, normalisation
methods based only on endogenous genes needed to be developed and we consider
them generally preferable, even for platforms where spike-ins may be used. Where
we have a large number of endogenous genes that, on average, do not vary
systematically between cells and where we expect technical effects to affect a
large number of genes (a very common and reasonable assumption), then such
methods (for example, the RUVs method) can perform well.

We explore both general approaches below.


```r
library(scRNA.seq.funcs)
library(RUVSeq)
library(scater)
library(SingleCellExperiment)
library(scran)
library(kBET)
library(sva) # Combat
library(edgeR)
library(harmony)
set.seed(1234567)
options(stringsAsFactors = FALSE)
umi <- readRDS("data/tung/umi.rds")
umi.qc <- umi[rowData(umi)$use, colData(umi)$use]
endog_genes <- !rowData(umi.qc)$is_feature_control
erccs <- rowData(umi.qc)$is_feature_control
## Apply scran sum factor normalization
qclust <- quickCluster(umi.qc, min.size = 30, use.ranks = FALSE)
umi.qc <- computeSumFactors(umi.qc, sizes = 15, clusters = qclust)
umi.qc <- normalize(umi.qc)
```


### Linear models

Linear models offer a relatively simple approach to accounting for batch effects
and confounders. A linear model can correct for batches while preserving
biological effects if you have a balanced design. In a confounded/replicate
design biological effects will not be fit/preserved. We could remove batch
effects from each individual separately in order to preserve biological (and
technical) variance between individuals (we will apply a similar with
`mnnCorrect`, below). 

Depending on how we have pre-processed our scRNA-seq data or what modelling
assumptions we are willing to make, we may choose to use normal (Gaussian)
linear models (i.e. assuming a normal distribution for noise) or generalized
linear models (GLM), where we can use any distribution from the exponential
family. Given that we obtain highly-variable count data from scRNA-seq assays,
the obvious choice for a GLM is to use the negative binomial distribution, which
has proven highly successful in the analysis of bulk RNA-seq data.

For demonstration purposes here we will naively correct all confounded batch
effects. 

#### Gaussian (normal) linear models

The [`limma`](https://bioconductor.org/packages/release/bioc/html/limma.html)
package in Bioconductor offers a convenient and efficient means to fit a linear
model (with the same design matrix) to a dataset with a large number of features
(i.e. genes) [@Ritchie2015-ra]. An added advantage of `limma` is its ability to
apply empirical Bayes squeezing of variance estimate to improve inference.

Provided we are satisfied making the assumption of a Gaussian distribution for
residuals (this may be reasonable for normalized log-counts in many cases; but
it may not be---debate continues in the literature), then we can apply `limma`
to regress out (known) unwanted sources of variation as follows.


```r
## fit a model just accounting for batch
lm_design_batch <- model.matrix(~0 + batch, data = colData(umi.qc))
fit_lm_batch <- lmFit(logcounts(umi.qc), lm_design_batch)
resids_lm_batch <- residuals(fit_lm_batch, logcounts(umi.qc))
assay(umi.qc, "lm_batch") <- resids_lm_batch

reducedDim(umi.qc, "PCA_lm_batch") <- reducedDim(
  runPCA(umi.qc[endog_genes, ], exprs_values = "lm_batch"), "PCA")

plotReducedDim(umi.qc, use_dimred = "PCA_lm_batch",
        colour_by = "batch", 
        size_by = "total_features_by_counts",
        shape_by = "individual"
        ) +
  ggtitle("LM - regress out batch")
```

Two problems are immediately apparent with the approach above. First, batch is
nested within individual, so simply regressing out batch as we have done above
also regresses out differences between individuals that we would like to
preserve. Second, we observe that the first principal component seems to
separate cells by number of genes (features) expressed, which is undesirable.

We can address these concerns by correcting for batch within each individual
separately, and also fitting the proportion of genes expressed per cell as a
covariate. [NB: to preserve overall differences in expression levels between
individuals we will need to apply a slight hack to the LM fit results (setting
the intercept coefficient to zero).]

__Exercise 2__

Perform LM correction for each individual separately. Store the final corrected
matrix in the `lm_batch_indi` slot.


```r
## define cellular detection rate (cdr), i.e. proportion of genes expressed in each cell
umi.qc$cdr <- umi.qc$total_features_by_counts_endogenous / nrow(umi.qc)
## fit a model just accounting for batch by individual
lm_design_batch1 <- model.matrix(~batch + cdr,
                                 data = colData(umi.qc)[umi.qc$individual == "na19098",])
fit_indi1 <- lmfit(logcounts(umi.qc)[, umi.qc$individual == "na19098"], lm_design_batch1)
fit_indi1$coefficients[,1] <- 0 ## replace intercept with 0 to preserve reference batch
resids_lm_batch1 <- residuals(fit_indi1, logcounts(umi.qc)[, umi.qc$individual == "na19098"])

lm_design_batch2 <- model.matrix(~batch + cdr,
                                 data = colData(umi.qc)[umi.qc$individual == "na19101",])
fit_indi2 <- lmfit(logcounts(umi.qc)[, umi.qc$individual == "na19101"], lm_design_batch2)
fit_indi2$coefficients[,1] <- 0 ## replace intercept with 0 to preserve reference batch
resids_lm_batch2 <- residuals(fit_indi2, logcounts(umi.qc)[, umi.qc$individual == "na19101"])

lm_design_batch3 <- model.matrix(~batch + cdr,
                                 data = colData(umi.qc)[umi.qc$individual == "na19239",])
fit_indi3 <- lmfit(logcounts(umi.qc)[, umi.qc$individual == "na19239"], lm_design_batch3)
fit_indi3$coefficients[,1] <- 0 ## replace intercept with 0 to preserve reference batch
resids_lm_batch3 <- residuals(fit_indi3, logcounts(umi.qc)[, umi.qc$individual == "na19239"])

identical(colnames(umi.qc), colnames(cbind(resids_lm_batch1, resids_lm_batch2, resids_lm_batch3)))
assay(umi.qc, "lm_batch_indi") <- cbind(resids_lm_batch1, resids_lm_batch2, resids_lm_batch3)

reduceddim(umi.qc, "pca_lm_batch_indi") <- reduceddim(
  runpca(umi.qc[endog_genes, ], exprs_values = "lm_batch_indi"), "pca")

plotreduceddim(umi.qc, use_dimred = "pca_lm_batch_indi",
        colour_by = "batch", 
        size_by = "total_features_by_counts",
        shape_by = "individual"
        ) +
  ggtitle("lm - regress out batch within individuals separately")
```

What do you think of the results of this approach?

#### Negative binomial generalized linear models

__Advanced exercise__

Can you use the `edgeR` package to use a negative binomial generalized linear
model to regress out batch effects?

_Hint_: follow a similar approach to that taken in the `limma` example above.
You will need to use the `DGEList()`, `estimateDisp()`, and `glmQLFit()`
functions.

### sctransform

The `sctransform` approach to using Pearson residuals from an regularized
negative binomial generalized linear model was introduced above. Here we
demonstrate how to apply this method.

Note that (due to what looks like a bug in this version of `sctransform`) we
need to convert the UMI count matrix to a sparse format to apply sctransform.

These `sctransform` results will face the problem mentioned above of batch being
nested within individual, which means that we cannot directly remove batch
effects without removing differences between individuals. However, here we will
demonstrate how you *would* try to remove batch effects with `sctransform` for a
kinder experimental design.


```r
umi_sparse <- as(counts(umi.qc), "dgCMatrix")
### Genes expressed in at least 5 cells will be kept
sctnorm_data <- sctransform::vst(umi = umi_sparse, min_cells = 1,
                                 cell_attr = as.data.frame(colData(umi.qc)),
                                 latent_var = c("log10_total_counts_endogenous", "batch"))
## Pearson residuals, or deviance residuals
sctnorm_data$model_str
assay(umi.qc, "sctrans_norm") <- sctnorm_data$y
```

Let us look at the NB GLM model parameters estimated by sctransform.


```r
#sce$log10_total_counts
##       Matrix of estimated model parameters per gene (theta and regression coefficients)
sctransform::plot_model_pars(sctnorm_data)
```

<img src="remove-conf_files/figure-html/sctransform-params-plot-1.png" width="90%" style="display: block; margin: auto;" />

Do these parameters and the regularization look sensible to you? Any concerns?


```r
reducedDim(umi.qc, "PCA_sctrans_norm") <- reducedDim(
  runPCA(umi.qc[endog_genes, ], exprs_values = "sctrans_norm")
)
plotReducedDim(
  umi.qc,
  use_dimred = "PCA_sctrans_norm",
  colour_by = "batch",
  size_by = "total_features_by_counts",
  shape_by = "individual"
) + ggtitle("PCA plot: sctransform normalization") 
```

<div class="figure" style="text-align: center">
<img src="remove-conf_files/figure-html/norm-pca-sctransform-1.png" alt="PCA plot of the tung data after sctransform normalisation (Pearson residuals)." width="90%" />
<p class="caption">(\#fig:norm-pca-sctransform)PCA plot of the tung data after sctransform normalisation (Pearson residuals).</p>
</div>

**Q:** What's happened here? Was that expected? Any other comments?


### Remove Unwanted Variation

Factors contributing to technical noise frequently appear as "batch effects"
where cells processed on different days or by different technicians
systematically vary from one another. Removing technical noise and correcting
for batch effects can frequently be performed using the same tool or slight
variants on it. We will be considering the [Remove Unwanted Variation
(RUVSeq)](http://bioconductor.org/packages/RUVSeq). Briefly, RUVSeq works as
follows. For $n$ samples and $J$ genes, consider the following generalized
linear model (GLM), where the RNA-Seq read counts are regressed on both the
known covariates of interest and unknown factors of unwanted variation: \[\log
E[Y|W,X,O] = W\alpha + X\beta + O\] Here, $Y$ is the $n \times J$ matrix of
observed gene-level read counts, $W$ is an $n \times k$ matrix corresponding to
the factors of “unwanted variation” and $O$ is an $n \times J$ matrix of offsets
that can either be set to zero or estimated with some other normalization
procedure (such as upper-quartile normalization). The simultaneous estimation of
$W$, $\alpha$, $\beta$, and $k$ is infeasible. For a given $k$, instead the
following three approaches to estimate the factors of unwanted variation $W$ are
used:

* _RUVg_ uses negative control genes (e.g. ERCCs), assumed to have constant
  expression across samples;
* _RUVs_ uses centered (technical) replicate/negative control samples for which
the covariates of interest are constant;
* _RUVr_ uses residuals, e.g., from a first-pass GLM regression of the counts on
  the covariates of interest.

We will concentrate on the first two approaches.

#### RUVg

To use _RUVg_ we will use ERCCs as negative control genes to anchor the
estimation of factors representing unwanted variation. _RUVg_ operates on the
raw count data. We adjust the output normalized counts from _RUVg_ so that they
represent normalized counts-per-million and then apply a log2 transformation. We
run _RUVg_ twice, with $k=1$ and $k=10$ so that we can compare the effect of
estimating different number of hidden factors to capture unwanted variation in
the data.


```r
ruvg <- RUVg(counts(umi.qc), erccs, k = 1)
assay(umi.qc, "ruvg1") <- log2(
    t(t(ruvg$normalizedCounts) / colSums(ruvg$normalizedCounts) * 1e6) + 1
)
ruvg <- RUVg(counts(umi.qc), erccs, k = 10)
assay(umi.qc, "ruvg10") <- log2(
    t(t(ruvg$normalizedCounts) / colSums(ruvg$normalizedCounts) * 1e6) + 1
)
```

When we assess the effectiveness of various batch correction methods below, you
can discuss whether or not you think using ERCCs as negative control genes for a
method like _RUVg_ is advisable (in this dataset and in general).

#### RUVs

In this application of _RUVs_ we treat the individuals as replicates for which
the covariates of interest are constant.

As above, we adjust the output normalized counts from _RUVs_ so that they
represent normalized counts-per-million and then apply a log2 transformation.
Again, we run the method with $k=1$ and $k=10$ so that we can compare the effect
of estimating different number of hidden factors.


```r
scIdx <- matrix(-1, ncol = max(table(umi.qc$individual)), nrow = 3)
tmp <- which(umi.qc$individual == "NA19098")
scIdx[1, 1:length(tmp)] <- tmp
tmp <- which(umi.qc$individual == "NA19101")
scIdx[2, 1:length(tmp)] <- tmp
tmp <- which(umi.qc$individual == "NA19239")
scIdx[3, 1:length(tmp)] <- tmp
cIdx <- rownames(umi.qc)
ruvs <- RUVs(counts(umi.qc), cIdx, k = 1, scIdx = scIdx, isLog = FALSE)
assay(umi.qc, "ruvs1") <- log2(
    t(t(ruvs$normalizedCounts) / colSums(ruvs$normalizedCounts) * 1e6) + 1
)
ruvs <- RUVs(counts(umi.qc), cIdx, k = 10, scIdx = scIdx, isLog = FALSE)
assay(umi.qc, "ruvs10") <- log2(
    t(t(ruvs$normalizedCounts) / colSums(ruvs$normalizedCounts) * 1e6) + 1
)
```

### Combat

If you have an experiment with a balanced design, `Combat` can be used to
eliminate batch effects while preserving biological effects by specifying the
biological effects using the `mod` parameter. However the `Tung` data contains
multiple experimental replicates rather than a balanced design so using `mod1`
to preserve biological variability will result in an error.


```r
combat_data <- logcounts(umi.qc)
mod_data <- as.data.frame(t(combat_data))
# Basic batch removal
mod0 <- model.matrix(~ 1, data = mod_data) 
# Preserve biological variability
mod1 <- model.matrix(~ umi.qc$individual, data = mod_data) 
# adjust for total genes detected
mod2 <- model.matrix(~ umi.qc$total_features_by_counts, data = mod_data)
assay(umi.qc, "combat") <- ComBat(
    dat = t(mod_data), 
    batch = factor(umi.qc$batch), 
    mod = mod0,
    par.prior = TRUE,
    prior.plots = FALSE
)
```

```
## Standardizing Data across genes
```

__Exercise 1__

Perform `ComBat` correction accounting for total features as a co-variate. Store the corrected matrix in the `combat_tf` slot.



### mnnCorrect 

`mnnCorrect` [@Haghverdi2017-vh] assumes that each batch shares at least one
biological condition with each other batch. Thus it works well for a variety of
balanced experimental designs. However, the `Tung` data contains multiple
replicates for each invidividual rather than balanced batches, thus we will
normalize each individual separately. Note that this will remove batch effects
between batches within the same individual but not the batch effects between
batches in different individuals, due to the confounded experimental design.

Thus we will merge a replicate from each individual to form three batches. 


```r
do_mnn <- function(data.qc) {
    batch1 <- logcounts(data.qc[, data.qc$replicate == "r1"])
    batch2 <- logcounts(data.qc[, data.qc$replicate == "r2"])
    batch3 <- logcounts(data.qc[, data.qc$replicate == "r3"])
    
    if (ncol(batch2) > 0) {
        x <- batchelor::mnnCorrect(
          batch1, batch2, batch3,  
          k = 20,
          sigma = 0.1,
          cos.norm.in = TRUE,
          svd.dim = 2
        )
        return(x)
    } else {
        x <- batchelor::mnnCorrect(
          batch1, batch3,  
          k = 20,
          sigma = 0.1,
          cos.norm.in = TRUE,
          svd.dim = 2
        )
        return(x)
    }
}

indi1 <- do_mnn(umi.qc[, umi.qc$individual == "NA19098"])
indi2 <- do_mnn(umi.qc[, umi.qc$individual == "NA19101"])
indi3 <- do_mnn(umi.qc[, umi.qc$individual == "NA19239"])

identical(colnames(umi.qc), colnames(cbind(indi1, indi2, indi3)))
```

```
## [1] TRUE
```

```r
assay(umi.qc, "mnn") <- assay(cbind(indi1, indi2, indi3), "corrected")

# For a balanced design: 
#assay(umi.qc, "mnn") <- mnnCorrect(
#    list(B1 = logcounts(batch1), B2 = logcounts(batch2), B3 = logcounts(batch3)),  
#    k = 20,
#    sigma = 0.1,
#    cos.norm = TRUE,
#    svd.dim = 2
#)
```

The latest version of the
[`batchelor`](https://www.bioconductor.org/packages/release/bioc/html/batchelor.html)
package has a new `fastMNN()` method. The `fastMNN()` function performs a
principal components (PCA). MNN identification and correction is preformed on
this low-dimensional representation of the data, an approach that offers some
advantages in speed and denoising. The function returns a `SingleCellExperiment`
object containing a matrix of corrected PC scores, which can be used directly
for downstream analyses like clustering and visualization. [NB: `fastMNN` may
actually be slower on small datasets like that considered here.]


```r
indi1 <- batchelor::fastMNN(
                      umi.qc[, umi.qc$individual == "NA19098"],
                      batch = umi.qc[, umi.qc$individual == "NA19098"]$replicate)
indi2 <- batchelor::fastMNN(
                      umi.qc[, umi.qc$individual == "NA19101"],
                      batch = umi.qc[, umi.qc$individual == "NA19101"]$replicate)
indi3 <- batchelor::fastMNN(
                      umi.qc[, umi.qc$individual == "NA19239"],
                      batch = umi.qc[, umi.qc$individual == "NA19239"]$replicate)

identical(colnames(umi.qc),
          colnames(cbind(assay(indi1, "reconstructed"),
                         assay(indi2, "reconstructed"),
                         assay(indi3, "reconstructed"))))    
```

```
## [1] TRUE
```

```r
fastmnn <- cbind(assay(indi1, "reconstructed"),
                 assay(indi2, "reconstructed"),
                 assay(indi3, "reconstructed"))
identical(rownames(umi.qc), rownames(fastmnn))
```

```
## [1] FALSE
```

```r
## fastMNN() drops 66 genes, so we cannot immediately add the reconstructed expression matrix to assays() in umi.qc
## But we can run PCA on the reconstructed data from fastMNN() and add that to the reducedDim slot of our SCE object
fastmnn_pca <- runPCA(fastmnn, rank=2)
reducedDim(umi.qc, "fastmnn") <- fastmnn_pca$rotation
```

For further details, please consult the `batchelor` package documentation and
[vignette](https://www.bioconductor.org/packages/release/bioc/vignettes/batchelor/inst/doc/correction.html).


### Harmony

Harmony [Korsunsky2018fast] is a newer batch correction method, which is
designed to operate on PC space. The algorithm proceeds to iteratively cluster
the cells, with the objective function formulated to promote cells from multiple
datasets within each cluster. Once a clustering is obtained, the positions of
the centroids of each dataset are obtained on a per-cluster basis and the
coordinates are corrected. This procedure is iterated until convergence. Harmony
comes with a `theta` parameter that controls the degree of batch correction
(higher values lead to more dataset integration), and can account for multiple
experimental and biological factors on input.

Seeing how the end result of Harmony is an altered dimensional reduction space
created on the basis of PCA, we plot the obtained manifold here and exclude it
from the rest of the follow-ups in the section.


```r
umi.qc.endog <- umi.qc[endog_genes,]
umi.qc.endog <- runPCA(umi.qc.endog, exprs_values = 'logcounts', ncomponents = 20)
pca <- as.matrix(reducedDim(umi.qc.endog, "PCA"))
harmony_emb <- HarmonyMatrix(pca, umi.qc.endog$batch, theta=2, do_pca=FALSE)
reducedDim(umi.qc.endog, "harmony") <- harmony_emb

plotReducedDim(
    umi.qc.endog,
    use_dimred = 'harmony',
    colour_by = "batch",
    size_by = "total_features_by_counts",
    shape_by = "individual"
)
```

<img src="remove-conf_files/figure-html/harmony-1.png" width="90%" style="display: block; margin: auto;" />

```r
reducedDim(umi.qc, "harmony") <- reducedDim(umi.qc.endog, "harmony")
```

### How to evaluate and compare batch correction

A key question when considering the different methods for removing confounders
is how to quantitatively determine which one is the most effective. The main
reason why comparisons are challenging is because it is often difficult to know
what corresponds to technical counfounders and what is interesting biological
variability. Here, we consider three different metrics which are all reasonable
based on our knowledge of the experimental design. Depending on the biological
question that you wish to address, it is important to choose a metric that
allows you to evaluate the confounders that are likely to be the biggest concern
for the given situation.

#### Effectiveness 1

We evaluate the effectiveness of the normalization by inspecting the
PCA plot where colour corresponds the technical replicates and shape
corresponds to different biological samples (individuals). Separation of biological samples and
interspersed batches indicates that technical variation has been
removed. We always use log2-cpm normalized data to match the assumptions of PCA.


```r
for (nm in assayNames(umi.qc)) {
  cat(nm, " \n")
  tmp <- runPCA(
    umi.qc[endog_genes, ],
    exprs_values = nm
  )
  reducedDim(umi.qc, paste0("PCA_", nm)) <- reducedDim(tmp, "PCA")
}
```

```
## counts  
## logcounts_raw  
## logcounts  
## sctrans_norm  
## ruvg1  
## ruvg10  
## ruvs1  
## ruvs10  
## combat  
## mnn
```

```r
for (nm in reducedDimNames(umi.qc)) {
  print(
    plotReducedDim(
      umi.qc,
      use_dimred = nm,
      colour_by = "batch",
      size_by = "total_features_by_counts",
      shape_by = "individual"
    ) +
    ggtitle(nm)
  )
}
```

<img src="remove-conf_files/figure-html/effectiveness1-1.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-2.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-3.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-4.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-5.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-6.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-7.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-8.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-9.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-10.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-11.png" width="90%" style="display: block; margin: auto;" /><img src="remove-conf_files/figure-html/effectiveness1-12.png" width="90%" style="display: block; margin: auto;" />

__Exercise 3__

Consider different `k`'s for RUV normalizations. Which gives the best results?

#### Effectiveness 2

We can also examine the effectiveness of correction using the relative log
expression (RLE) across cells to confirm technical noise has been removed from
the dataset. Note RLE only evaluates whether the number of genes higher and
lower than average are equal for each cell - i.e. systemic technical effects.
Random technical noise between batches may not be detected by RLE.


```r
res <- list()
for(n in assayNames(umi.qc)) {
	res[[n]] <- suppressWarnings(calc_cell_RLE(assay(umi.qc, n), erccs))
}
par(mar=c(6,4,1,1))
boxplot(res, las=2)
```

<img src="remove-conf_files/figure-html/effectiveness2-1.png" width="90%" style="display: block; margin: auto;" />

#### Effectiveness 3

Another method to check the efficacy of batch-effect correction is to consider
the intermingling of points from different batches in local subsamples of the
data. If there are no batch-effects then proportion of cells from each batch in
any local region should be equal to the global proportion of cells in each
batch.

`kBET` [@Buttner2017-ds] takes `kNN` networks around random cells and tests the
number of cells from each batch against a binomial distribution. The rejection
rate of these tests indicates the severity of batch-effects still present in the
data (high rejection rate = strong batch effects). `kBET` assumes each batch
contains the same complement of biological groups, thus it can only be applied
to the entire dataset if a perfectly balanced design has been used. However,
`kBET` can also be applied to replicate-data if it is applied to each biological
group separately. In the case of the Tung data, we will apply `kBET` to each
individual independently to check for residual batch effects. However, this
method will not identify residual batch-effects which are confounded with
biological conditions. In addition, `kBET` does not determine if biological
signal has been preserved.


```r
compare_kBET_results <- function(sce){
    indiv <- unique(sce$individual)
    norms <- assayNames(sce) # Get all normalizations
    results <- list()
    for (i in indiv){ 
        for (j in norms){
            tmp <- kBET(
                df = t(assay(sce[,sce$individual== i], j)), 
                batch = sce$batch[sce$individual==i], 
                heuristic = TRUE, 
                verbose = FALSE, 
                addTest = FALSE, 
                plot = FALSE)
            results[[i]][[j]] <- tmp$summary$kBET.observed[1]
        }
    }
    return(as.data.frame(results))
}

eff_debatching <- compare_kBET_results(umi.qc)
```


```r
require("reshape2")
require("RColorBrewer")
# Plot results
dod <- melt(as.matrix(eff_debatching),  value.name = "kBET")
colnames(dod)[1:2] <- c("Normalisation", "Individual")

colorset <- c('gray', brewer.pal(n = 9, "Oranges"))

ggplot(dod, aes(Normalisation, Individual, fill=kBET)) +  
  geom_tile() +
  scale_fill_gradient2(
    na.value = "gray",
    low = colorset[2],
    mid=colorset[6],
    high = colorset[10],
    midpoint = 0.5, limit = c(0,1)) +
  scale_x_discrete(expand = c(0, 0)) +
  scale_y_discrete(expand = c(0, 0)) + 
  theme(
    axis.text.x = element_text(
      angle = 45, 
      vjust = 1, 
      size = 12, 
      hjust = 1
    )
  ) + 
  ggtitle("Effect of batch regression methods per individual")
```

<img src="remove-conf_files/figure-html/kbet-1.png" width="90%" style="display: block; margin: auto;" />

__Exercise 4__

Why do the raw counts appear to have little batch effects?

### Big Exercise

Perform the same analysis with read counts of the `tung` data. Use
`tung/reads.rds` file to load the reads `SCE` object. Once you have finished
please compare your results to ours (next chapter). Additionally, experiment
with other combinations of normalizations and compare the results.

### sessionInfo()


```
## R version 3.6.0 (2019-04-26)
## Platform: x86_64-pc-linux-gnu (64-bit)
## Running under: Ubuntu 18.04.3 LTS
## 
## Matrix products: default
## BLAS:   /usr/lib/x86_64-linux-gnu/blas/libblas.so.3.7.1
## LAPACK: /usr/lib/x86_64-linux-gnu/lapack/liblapack.so.3.7.1
## 
## locale:
##  [1] LC_CTYPE=en_AU.UTF-8       LC_NUMERIC=C              
##  [3] LC_TIME=en_AU.UTF-8        LC_COLLATE=en_AU.UTF-8    
##  [5] LC_MONETARY=en_AU.UTF-8    LC_MESSAGES=en_AU.UTF-8   
##  [7] LC_PAPER=en_AU.UTF-8       LC_NAME=C                 
##  [9] LC_ADDRESS=C               LC_TELEPHONE=C            
## [11] LC_MEASUREMENT=en_AU.UTF-8 LC_IDENTIFICATION=C       
## 
## attached base packages:
## [1] stats4    parallel  stats     graphics  grDevices utils     datasets 
## [8] methods   base     
## 
## other attached packages:
##  [1] RColorBrewer_1.1-2          reshape2_1.4.3             
##  [3] harmony_1.0                 Rcpp_1.0.2                 
##  [5] sva_3.32.1                  genefilter_1.66.0          
##  [7] mgcv_1.8-28                 nlme_3.1-139               
##  [9] kBET_0.99.6                 scran_1.12.1               
## [11] scater_1.12.2               ggplot2_3.2.1              
## [13] SingleCellExperiment_1.6.0  RUVSeq_1.18.0              
## [15] edgeR_3.26.8                limma_3.40.6               
## [17] EDASeq_2.18.0               ShortRead_1.42.0           
## [19] GenomicAlignments_1.20.1    SummarizedExperiment_1.14.1
## [21] DelayedArray_0.10.0         matrixStats_0.55.0         
## [23] Rsamtools_2.0.1             GenomicRanges_1.36.1       
## [25] GenomeInfoDb_1.20.0         Biostrings_2.52.0          
## [27] XVector_0.24.0              IRanges_2.18.3             
## [29] S4Vectors_0.22.1            BiocParallel_1.18.1        
## [31] Biobase_2.44.0              BiocGenerics_0.30.0        
## [33] scRNA.seq.funcs_0.1.0      
## 
## loaded via a namespace (and not attached):
##   [1] backports_1.1.4          aroma.light_3.14.0      
##   [3] plyr_1.8.4               igraph_1.2.4.1          
##   [5] lazyeval_0.2.2           splines_3.6.0           
##   [7] listenv_0.7.0            elliptic_1.4-0          
##   [9] digest_0.6.21            htmltools_0.3.6         
##  [11] viridis_0.5.1            magrittr_1.5            
##  [13] memoise_1.1.0            contfrac_1.1-12         
##  [15] cluster_2.1.0            globals_0.12.4          
##  [17] annotate_1.62.0          R.utils_2.9.0           
##  [19] prettyunits_1.0.2        colorspace_1.4-1        
##  [21] blob_1.2.0               xfun_0.9                
##  [23] dplyr_0.8.3              crayon_1.3.4            
##  [25] RCurl_1.95-4.12          zeallot_0.1.0           
##  [27] survival_2.43-3          glue_1.3.1              
##  [29] gtable_0.3.0             zlibbioc_1.30.0         
##  [31] BiocSingular_1.0.0       future.apply_1.3.0      
##  [33] scales_1.0.0             DESeq_1.36.0            
##  [35] DBI_1.0.0                viridisLite_0.3.0       
##  [37] xtable_1.8-4             progress_1.2.2          
##  [39] dqrng_0.2.1              bit_1.1-14              
##  [41] rsvd_1.0.2               deSolve_1.24            
##  [43] httr_1.4.1               FNN_1.1.3               
##  [45] pkgconfig_2.0.3          XML_3.98-1.20           
##  [47] R.methodsS3_1.7.1        locfit_1.5-9.1          
##  [49] dynamicTreeCut_1.63-1    tidyselect_0.2.5        
##  [51] labeling_0.3             rlang_0.4.0             
##  [53] AnnotationDbi_1.46.1     munsell_0.5.0           
##  [55] tools_3.6.0              moments_0.14            
##  [57] RSQLite_2.1.2            batchelor_1.0.1         
##  [59] evaluate_0.14            stringr_1.4.0           
##  [61] yaml_2.2.0               knitr_1.25              
##  [63] bit64_0.9-7              hypergeo_1.2-13         
##  [65] purrr_0.3.2              future_1.14.0           
##  [67] R.oo_1.22.0              biomaRt_2.40.4          
##  [69] compiler_3.6.0           beeswarm_0.2.3          
##  [71] tibble_2.1.3             statmod_1.4.32          
##  [73] geneplotter_1.62.0       stringi_1.4.3           
##  [75] highr_0.8                GenomicFeatures_1.36.4  
##  [77] lattice_0.20-38          Matrix_1.2-17           
##  [79] vctrs_0.2.0              lifecycle_0.1.0         
##  [81] pillar_1.4.2             BiocNeighbors_1.2.0     
##  [83] cowplot_1.0.0            bitops_1.0-6            
##  [85] orthopolynom_1.0-5       irlba_2.3.3             
##  [87] rtracklayer_1.44.4       R6_2.4.0                
##  [89] latticeExtra_0.6-28      hwriter_1.3.2           
##  [91] bookdown_0.13            gridExtra_2.3           
##  [93] vipor_0.4.5              codetools_0.2-16        
##  [95] MASS_7.3-51.1            assertthat_0.2.1        
##  [97] withr_2.1.2              sctransform_0.2.0       
##  [99] GenomeInfoDbData_1.2.1   hms_0.5.1               
## [101] grid_3.6.0               tidyr_1.0.0             
## [103] rmarkdown_1.15           DelayedMatrixStats_1.6.1
## [105] Rtsne_0.15               ggbeeswarm_0.6.0
```

